/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Application.h"

#include "ConfigurationDialog.h"
#include "Debug.h"
#include "DisplayWidget.h"
#include "DuplicateImageDialog.h"
#include "File.h"
#include "FileDialog.h"
#include "IconEngine.h"
#include "IconNames.h"
#include "IconEngine.h"
#include "ImageRecordList.h"
#include "ImportImageDialog.h"
#include "InformationDialog.h"
#include "MainWindow.h"
#include "QuestionDialog.h"
#include "XmlDocument.h"
#include "XmlOptions.h"

#include <QMessageBox>
#include <QApplication>

//____________________________________________
Application::Application( CommandLineArguments arguments ) :
    BaseApplication( nullptr, arguments ),
    Counter( "Application" )
{}

//____________________________________________
Application::~Application()
{
    Debug::Throw( "Application::~Application.\n" );
    if( mainWindow_ ) delete mainWindow_;
    if( recentImages_ ) delete recentImages_;
}

//____________________________________________
bool Application::initApplicationManager()
{
    Debug::Throw( "Application::initApplicationManager.\n" );

    // retrieve files from arguments and expand if needed
    CommandLineParser parser( commandLineParser( _arguments() ) );
    for( auto& orphan:parser.orphans() )
    { if( !orphan.isEmpty() ) File::expand( orphan ); }

    // replace arguments
    _setArguments( parser.arguments() );

    // base class initialization
    return BaseApplication::initApplicationManager();

}

//____________________________________________
bool Application::realizeWidget()
{
    Debug::Throw( "Application::realizeWidget.\n" );

    // check if the method has already been called.
    if( !BaseApplication::realizeWidget() ) return false;

    // need to redirect closeAction to proper exit
    closeAction().disconnect();
    connect( &closeAction(), SIGNAL(triggered()), SLOT(_exit()) );

    // actions
    _installActions();

    // recent files
    recentImages_ = new ImageRecordList( this );
    recentImages_->setCheck( true );
    recentImages_->checkValidFiles();

    // create selection frame
    mainWindow_ = new MainWindow;

    // update configuration
    emit configurationChanged();

    mainWindow_->centerOnDesktop();
    mainWindow_->show();

    // update
    qApp->processEvents();

    // load file from arguments or recent files
    QStringList filenames( commandLineParser( _arguments() ).orphans() );

    if( !filenames.empty() ) mainWindow_->setImageRecord( recentImages().get( File( filenames.front() ).expand() ) );
    else {

        ImageRecord record( recentImages().lastValidFile() );
        if( !record.file().file().isEmpty() ) mainWindow_->setImageRecord( record );
        else _open();

    }

    return true;

}

//____________________________________________
void Application::usage() const
{
    _usage( "graphscan", tr( "[options] [file]" ) );
    commandLineParser().usage();
    return;
}

//_________________________________________________
void Application::open( ImageRecord record )
{

    Debug::Throw()
        << "Application::open -"
        << " file: " << record.file()
        << " version: " << record.version()
        << endl;

    // save current record
    if( mainWindow_->hasImageRecord() )
    {
        const ImageRecord record( mainWindow_->imageRecord() );
        recentImages().add( mainWindow_->imageRecord() );
        recentImages().write();
    }

    // open new record
    mainWindow_->setImageRecord( recentImages().get( record.file().file(), record.version() ) );

}

//_________________________________________________
void Application::_configuration()
{

    Debug::Throw( "Application::_configuration" );
    emit saveConfiguration();

    ConfigurationDialog dialog;
    connect( &dialog, SIGNAL(configurationChanged()), SIGNAL(configurationChanged()) );
    dialog.centerOnWidget( qApp->activeWindow() );
    dialog.exec();

}

//_________________________________________________
void Application::_open()
{

    Debug::Throw( "Application::_open.\n" );

    FileDialog dialog( mainWindow_ );
    if( mainWindow_->hasImageRecord() )
    { dialog.selectFile( mainWindow_->displayWidget().file() ); }

    auto file( dialog.getFile() );
    if( !file.isEmpty() ) {

        // check if found in recent images
        if( !recentImages().contains( file, 0 ) )
        {

            open( ImageRecord( file, 0 ) );

        } else {

            // find last version
            int version = 0;
            for( ; recentImages().contains( file, version ); version++ )
            {}

            // create dialog
            DuplicateImageDialog dialog( mainWindow_, file );
            if( XmlOptions::get().contains( "DUPLICATION_MODE" ) ) dialog.setMode( (DuplicateImageDialog::Mode) XmlOptions::get().get<int>( "DUPLICATION_MODE" ) );
            if( !dialog.exec() ) return;

            XmlOptions::get().set<DuplicateImageDialog::Mode>( "DUPLICATION_MODE", dialog.mode() );
            open( ImageRecord( file, dialog.mode() == DuplicateImageDialog::OpenAgain ? version-1:version ) );

        }

    }

}

//_________________________________________________
void Application::_save()
{

    Debug::Throw( "Application::_save.\n" );
    if( !mainWindow_->hasImageRecord() ) return;

    const ImageRecord record( mainWindow_->imageRecord() );
    Debug::Throw() << "Application::open - file: " << record.file().file() << " refPoints: " << record.refPoints().size() << endl;

    recentImages().add( record );
    recentImages().write();
}

//_________________________________________________
void Application::_reload()
{

    Debug::Throw( "Application::_reload.\n" );
    if( !mainWindow_->hasImageRecord() ) return;

    FileRecord file( mainWindow_->imageRecord().file() );
    const QString buffer = QString( tr( "Discard changes to '%1' ?" ) ).arg( file.file().localName() );
    if( !QuestionDialog( mainWindow_, buffer ).exec() )
    { return; }

    ImageRecord record( recentImages().get( file.file() ) );
    mainWindow_->setImageRecord( record );

}

//_________________________________________________
void Application::_exit()
{
    Debug::Throw( "Application::_exit.\n" );
    qApp->quit();
}

//_________________________________________________
void Application::_import()
{
    Debug::Throw( "Application::_import.\n" );

    FileDialog dialog( mainWindow_ );
    if( mainWindow_->hasImageRecord() )
    {
        const File file( mainWindow_->imageRecord().file().file().truncatedName() + ".xml" );
        if( file.exists() ) dialog.selectFile( file );
    }

    // get file and check
    QString input( dialog.getFile() );
    if( input.isNull() ) return;
    QFile in( input );
    if ( !in.open( QIODevice::ReadOnly ) )
    {
        Debug::Throw( "ImageRecordList::_read - cannot open file.\n" );
        return;
    }

    // get file record
    XmlDocument document;
    if ( !document.setContent( &in ) )
    {
        Debug::Throw() << document.error() << endl;
        return;
    }

    auto records = ImageRecord::Helper::list( document.get().documentElement() );
    if( records.size() != 1 ) return;

    ImageRecord record( records.front() );

    // check records
    if( !mainWindow_->hasImageRecord() )
    {
        mainWindow_->setImageRecord( record );
        return;
    }

    // store file
    const File currentFile( mainWindow_->imageRecord().file().file() );
    const File newFile( record.file().file() );

    if( newFile.exists() ) {

        ImportImageDialog::Modes mask( ImportImageDialog::ReplaceCurrent | ImportImageDialog::Create );
        if( newFile != currentFile && recentImages().contains( newFile ) ) mask |= ImportImageDialog::ReplaceExisting;

        ImportImageDialog dialog( mainWindow_, currentFile, mask );
        if( XmlOptions::get().contains( "IMPORT_MODE" ) && (mask&XmlOptions::get().get<int>( "IMPORT_MODE" )) )
        { dialog.setMode( (ImportImageDialog::Mode) XmlOptions::get().get<int>( "IMPORT_MODE" ) ); }

        // show dialog and check
        if( !dialog.exec() ) return;

        // save choice
        XmlOptions::get().set<int>( "IMPORT_MODE", dialog.mode() );

        if( dialog.mode() == ImportImageDialog::Create )
        {

            // save current
            if( mainWindow_->hasImageRecord() )
            {
                ImageRecord record( mainWindow_->imageRecord() );
                recentImages().add( mainWindow_->imageRecord() );
                recentImages().write();
            }

            // find last version
            int version = 0;
            for( ; recentImages().contains( newFile, version ); version++ )
            {}

            record.setVersion( version );
            mainWindow_->setImageRecord( recentImages().add( record ) );

        } else if( dialog.mode() == ImportImageDialog::ReplaceCurrent ) {

            // copy file and version to new record and assign
            record.setFile( currentFile );
            record.setVersion( mainWindow_->imageRecord().version() );
            mainWindow_->setImageRecord( recentImages().add( record ) );

        } else if( dialog.mode() == ImportImageDialog::ReplaceExisting ) {

            // save current
            if( mainWindow_->hasImageRecord() )
            {
                ImageRecord record( mainWindow_->imageRecord() );
                recentImages().add( mainWindow_->imageRecord() );
                recentImages().write();
            }

            // find last version
            int version = 0;
            for( ; recentImages().contains( newFile, version ); version++ )
            {}

            record.setVersion( version-1 );
            mainWindow_->setImageRecord( recentImages().add( record ) );

        }

    } else {

        const QString buffer = QString(
            tr( "Imported file '%1' does not match the current file."
            "Do you want to replace the current data anyway ?" ) ).arg( newFile );
        if( !QuestionDialog( mainWindow_, buffer ).exec() ) return;

        // copy file and version to new record and assign
        record.setFile( currentFile );
        record.setVersion( mainWindow_->imageRecord().version() );
        mainWindow_->setImageRecord( recentImages().add( record ) );

    }

}

//_________________________________________________
void Application::_export()
{
    Debug::Throw( "Application::_export.\n" );

    // check validity
    if( !mainWindow_->hasImageRecord() ) return;

    // get record
    const ImageRecord record( mainWindow_->imageRecord() );

    // generate file
    const File file( record.file().file().truncatedName() + ".xml" );
    FileDialog dialog( mainWindow_ );
    dialog.setAcceptMode( QFileDialog::AcceptSave );
    dialog.selectFile( file );

    // get output and check
    QString output( dialog.getFile() );
    if( output.isNull() ) return;

    // output file
    QFile out( output );
    if( !out.open( QIODevice::WriteOnly ) ) return;

    // create document
    QDomDocument document;

    // append truncated list
    document.appendChild( ImageRecord::Helper::domElement( { record }, document ) );

    // wrie and close
    out.write( document.toByteArray() );
    out.close();

}

//________________________________________________
bool Application::_processCommand( Server::ServerCommand command )
{

    Debug::Throw( "Application::_processCommand.\n" );
    if( BaseApplication::_processCommand( command ) ) return true;
    return false;

}

//_________________________________________________
void Application::_aboutToQuit()
{
    Debug::Throw( "Application::_aboutToQuit.\n" );
    BaseApplication::_aboutToQuit();

    if( mainWindow_->hasImageRecord() )
    { recentImages().add( mainWindow_->imageRecord() ); }

}

//_________________________________________________
void Application::_installActions()
{
    Debug::Throw( "Application::_installActions.\n" );

    openAction_ = new QAction( IconEngine::get( IconNames::Open ), tr( "Open" ), this );
    openAction_->setShortcut( QKeySequence::Open );
    connect( openAction_, SIGNAL(triggered()), SLOT(_open()) );

    saveAction_ = new QAction( IconEngine::get( IconNames::Save ), tr( "Save" ), this );
    saveAction_->setShortcut( QKeySequence::Save );
    connect( saveAction_, SIGNAL(triggered()), SLOT(_save()) );

    reloadAction_ = new QAction( IconEngine::get( IconNames::Reload ), tr( "Reload" ), this );
    reloadAction_->setShortcut( QKeySequence::Refresh );
    connect( reloadAction_, SIGNAL(triggered()), SLOT(_reload()) );

    importAction_ = new QAction( IconEngine::get( IconNames::Import ), tr( "Import" ), this );
    importAction_->setShortcut( Qt::SHIFT + Qt::CTRL + Qt::Key_I );
    connect( importAction_, SIGNAL(triggered()), SLOT(_import()) );

    exportAction_ = new QAction( IconEngine::get( IconNames::Export ), tr( "Export" ), this );
    exportAction_->setShortcut( Qt::SHIFT + Qt::CTRL + Qt::Key_E );
    connect( exportAction_, SIGNAL(triggered()), SLOT(_export()) );
}
