#ifndef Application_h
#define Application_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "BaseApplication.h"
#include "CommandLineArguments.h"
#include "CommandLineParser.h"
#include "Config.h"
#include "Counter.h"
#include "IconEngine.h"
#include "ImageRecord.h"

class ImageRecordList;
class MainWindow;

/**
\class  Application
\brief  Main Window singleton object
*/

class Application: public BaseApplication, private Base::Counter<Application>
{

    //* Qt meta object declaration
    Q_OBJECT

    public:

    //* constructor
    explicit Application( CommandLineArguments );

    //* destructor
    ~Application();

    //* application manager
    bool initApplicationManager() override;

    //* create all widgets
    bool realizeWidget() override;

    //* file list
    ImageRecordList& recentImages() const
    { return *recentImages_; }

    //* retrieve MainWindow singleton
    MainWindow& mainWindow() const
    { return *mainWindow_; }

    //*@name actions
    //@{

    //* open
    QAction& openAction() const
    { return *openAction_; }

    //* reload
    QAction& reloadAction() const
    { return *reloadAction_; }

    //* save
    QAction& saveAction() const
    { return *saveAction_; }

    //* import
    QAction& importAction() const
    { return *importAction_; }

    //* export
    QAction& exportAction() const
    { return *exportAction_; }

    //@}

    //*@name application information
    //@{

    //* command line help
    void usage() const override;

    //* application name
    QString applicationName() const override
    { return "Graphscan"; }

    //* application icon
    QIcon applicationIcon() const override
    { return IconEngine::get( ":/graphscan.png" ); }

    // application version
    QString applicationVersion() const override
    { return VERSION; }

    //@}

    public Q_SLOTS:

    //* open
    void open( ImageRecord );

    protected Q_SLOTS:

    //* configuration
    void _configuration() override;

    //* open
    void _open();

    //* save
    void _save();

    //* reload
    void _reload();

    //* exit safely
    void _exit();

    //* open data
    void _import();

    //* export data
    void _export();

    //* process request from application manager
    bool _processCommand( Server::ServerCommand ) override;

    //* about to quit
    void _aboutToQuit() override;

    protected:

    //* install actions
    void _installActions();

    private:

    //* recent files
    ImageRecordList* recentImages_ = nullptr;

    //* main window entry selection frame
    MainWindow* mainWindow_ = nullptr;

    //*@name actions
    //@{

    //* open
    QAction* openAction_ = nullptr;

    //* save
    QAction* saveAction_ = nullptr;

    //* reload
    QAction* reloadAction_ = nullptr;

    //* import action
    QAction* importAction_ = nullptr;

    //* export action
    QAction* exportAction_ = nullptr;

    //@}

};

#endif
