#ifndef DisplayWidget_h
#define DisplayWidget_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "File.h"
#include "RefPoint.h"
#include "ZoomWidget.h"

#include <QDialog>
#include <QMenu>
#include <QPaintEvent>
#include <QPixmap>
#include <QWidget>

#include <QBasicTimer>

class DisplayWidget: public QWidget, private Base::Counter<DisplayWidget>
{

    Q_OBJECT

    public:

    //* constructor
    explicit DisplayWidget( QWidget* = nullptr );

    //* set image
    void setImage( const File& );

    //* file
    const File& file() const
    { return file_; }

    //* clear
    void clear()
    {
        file_ = File();
        refPoints_.clear();
        points_.clear();
        selectedPoints_.clear();
        hoverMode_ = RefPoint::NoHover;
        dragMode_ = RefPoint::NoDrag;
        update();
    }

    //* ref points
    void setRefPoints( const RefPoint::Set& value )
    {
        refPoints_ = value;
        update();
    }

    //* ref points validity
    void setRefPointsValid( bool value )
    { refPointsValid_ = value; }

    //* collection
    void setCollection( const RefPoint::Collection& collection )
    { dataSets_ = collection; }

    //* index
    void setDataSetIndex( int value )
    { dataSetIndex_ = value; }

    //* points
    void setPoints( const RefPoint::Set& );

    //* restore drag mode
    void setDragMode( RefPoint::DragMode dragMode )
    { dragMode_ = dragMode; }

    //* add single point
    void addPoint( const RefPoint& value )
    {
        points_.get().insert( value );
        _updateHoverPoint();
        update();
    }

    //* selected points
    void clearSelection()
    {
        selectedPoints_.clear();
        update();
    }

    //* selected points
    void setSelection( const RefPoint& value )
    {
        selectedPoints_.clear();
        selectedPoints_.get().insert( value );
        update();
    }

    //* selected points
    void setSelection( const RefPoint::List& value )
    {
        selectedPoints_.clear();
        for( const auto& point:value.get() )
        { selectedPoints_.get().insert( point ); }
        update();
    }

    //* selected points
    void setSelection( const RefPoint::Set& value )
    {
        selectedPoints_ = value;
        update();
    }

    //*@name actions
    //@{

    QAction& showRefPointsAction() const
    { return *showRefPointsAction_; }

    QAction& showInactivePointsAction() const
    { return *showInactivePointsAction_; }

    QAction& showScanLinesAction() const
    { return *showScanLinesAction_; }

    QAction& addPointAction() const
    { return *addPointAction_; }

    QAction& removePointAction() const
    { return *removePointAction_; }

    QAction& copyPointAction() const
    { return *copyPointAction_; }

    QAction& pastePointAction() const
    { return *pastePointAction_; }

    QAction& editErrorsAction() const
    { return *editErrorsAction_; }

    QAction& defaultErrorsAction() const
    { return *defaultErrorsAction_; }

    QAction& defaultErrorsSystAction() const
    { return *defaultErrorsSystAction_; }

    //@}

    //* event handling
    bool eventFilter( QObject*, QEvent* ) override;

    //* true if current position is valid
    bool positionIsValid();

    //* hover point
    const RefPoint& hoverPoint() const
    { return hoverPoint_; }

    //* hover point
    void setHoverPoint( const RefPoint& value )
    { hoverPoint_ = value; }

    //* hover mode
    RefPoint::HoverMode hoverMode() const
    { return hoverMode_; }

    //* drag point
    const RefPoint& dragPoint() const
    { return dragPoint_; }

    //* drag point
    void setDragPoint( const RefPoint& value )
    { dragPoint_ = value; }

    //* drag mode
    RefPoint::DragMode dragMode() const
    { return dragMode_; }

    //* real zoom factor
    double realZoomFactor() const
    { return realZoomFactor_; }

    Q_SIGNALS:

    //* mouse position changed
    void pointerMoved( QPoint, RefPoint::HoverMode );

    //* mouse left widget
    void widgetEntered();

    //* mouse left widget
    void widgetLeft();

    //* real factor
    void realZoomFactorChanged( double );

    //* point selected
    void pointSelected();

    //* point added
    void pointAdded( QPointF );

    //* point pasted
    void pointPasted( QPointF );

    //* point dragged
    void pointDragged( QPointF, RefPoint::DragMode );

    //* remove selected markers
    void selectionRemoved( const RefPoint::List& );

    public Q_SLOTS:

    //* zoom modified function
    void zoomModified( ZoomWidget::Mode, double );

    //* update position
    void updatePosition();

    //* update mouse position
    bool setPosition( QPoint );

    protected:

    //*@name event handling
    //@{
    bool event( QEvent* ) override;
    void paintEvent( QPaintEvent* ) override;
    void timerEvent( QTimerEvent* ) override;
    void mouseMoveEvent( QMouseEvent* ) override;
    void mousePressEvent( QMouseEvent* ) override;
    void mouseReleaseEvent( QMouseEvent* ) override;
    void contextMenuEvent( QContextMenuEvent* ) override;
    //@}

    //* adjust zoom factor
    void _adjustZoomFactor();

    //* scaled pixmap
    void _updateScaledPixmap();

    //* update hover point
    bool _updateHoverPoint();

    //* draw data set
    void _drawDataSet( QPainter&, int ) const;

    //* draw symbol
    void _drawSymbol( QPainter&, const RefPoint&, const RefPoint::HoverMode& = RefPoint::NoHover ) const;

    //* find index of nearby point, if any
    RefPoint _findPoint( const QPointF&, RefPoint::HoverMode& ) const;

    class FindQPointFTor
    {

        public:

        //* constructor
        explicit FindQPointFTor( QPointF position, const double& pointSize ):
            position_( position ),
            pointSize_( pointSize ),
            minDistance_( -1 ),
            found_( false ),
            vectXInitialized_( false ),
            vectYInitialized_( false )
        {}

        //* hover mode for given ref point
        RefPoint::HoverMode hoverMode( const RefPoint& );

        //* true if one point was found
        bool found() const
        { return found_; }

        //* found point
        const RefPoint& out()
        { return out_; }

        private:

        //* predicted position (scaled)
        QPointF position_;

        //* point size (scaled)
        double pointSize_;

        //* min distance to point center
        double minDistance_;

        //* true if point is found
        bool found_;

        //* found point
        RefPoint out_;

        //* true if X vector is initialized
        bool vectXInitialized_;

        //* true if Y vector is initialized
        bool vectYInitialized_;

        //* X vector
        QPointF vectX_;

        //* Y vector
        QPointF vectY_;

    };

    protected Q_SLOTS:

    //* toggle show ref point
    void _toggleShowRefPoints( bool );

    //* toggle show ref point
    void _toggleShowInactivePoints( bool );

    //* toggle show scan lines
    void _toggleShowScanLines( bool );

    //* add point
    void _addPoint();

    //* paste point
    void _pastePoint();

    //* remove selection
    void _removeSelection();

    private Q_SLOTS:

    //* update configuration
    void _updateConfiguration();

    private:

    //* install actions
    void _installActions();

    //* image
    File file_;

    //* pixmap
    QPixmap pixmap_;

    //* pixmap
    QPixmap scaledPixmap_;

    //* current position
    QPoint position_;

    //* click position
    QPoint clickPosition_;

    //* reference points
    RefPoint::Set refPoints_;

    //* ref point validity
    bool refPointsValid_ = false;

    //* collection
    RefPoint::Collection dataSets_;

    //* collection index
    int dataSetIndex_ = 0;

    //* points
    RefPoint::Set points_;

    //* selected points
    RefPoint::Set selectedPoints_;

    //*@name options
    //@{

    double pointSize_ = 20;
    double penWidth_ = 1;
    QColor inactivePointColor_;
    QColor pointColor_;
    QColor refPointColor_;
    QColor selectedPointColor_;
    QColor hoverPointColor_;

    //@}

    //*@name actions
    //@{

    QAction* showRefPointsAction_ = nullptr;
    QAction* showInactivePointsAction_ = nullptr;
    QAction* showScanLinesAction_ = nullptr;
    QAction* addPointAction_ = nullptr;
    QAction* removePointAction_ = nullptr;
    QAction* copyPointAction_ = nullptr;
    QAction* pastePointAction_ = nullptr;
    QAction* editErrorsAction_ = nullptr;
    QAction* defaultErrorsAction_ = nullptr;
    QAction* defaultErrorsSystAction_ = nullptr;

    //@}

    //* resize timer
    QBasicTimer resizeTimer_;

    //*@name zoom
    //@{
    ZoomWidget::Mode zoomMode_ = ZoomWidget::Fixed;
    double zoomFactor_ = 1.0;
    double realZoomFactor_ = 1.0;
    //@}

    //*@name hover
    //@{
    RefPoint::HoverMode hoverMode_ = RefPoint::NoHover;
    RefPoint hoverPoint_;
    //@}

    //*@name drag
    //@{
    RefPoint::DragMode dragMode_ = RefPoint::NoDrag;
    RefPoint dragPoint_;
    //@}

};

#endif
