/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ImageRecord.h"
#include "XmlFileRecord.h"
#include "XmlDef.h"

//_____________________________________________________________________________
ImageRecord::ImageRecord( const File& file, int version ):
    Counter( "ImageRecord" ),
    file_( file ),
    version_( version )
{
    // make sure there is at least one data set
    dataSets_ << RefPoint::List();
}

//_____________________________________________________________________________
ImageRecord::ImageRecord( const QDomElement& element ):
    Counter( "ImageRecord" )
{
    Debug::Throw( "ImageRecord::ImageRecord.\n" );

    QDomNamedNodeMap attributes( element.attributes() );
    for( int i=0; i<attributes.count(); i++ )
    {
        QDomAttr attribute( attributes.item( i ).toAttr() );
        if( attribute.isNull() ) continue;

        QString name( attribute.name() );
        QString value( attribute.value() );
        if( name == Xml::Flags ) flags_ = Flags( value.toInt() );
        else if( name == Xml::Index ) dataSetIndex_ = value.toInt();
        else if( name == Xml::ImageVersion ) version_ = value.toUInt();
        else Debug::Throw(0) << "ImageRecord::ImageRecord - unrecognized entry attribute: \"" << name << "\"" << endl;

    }

    // parse children elements
    for(QDomNode childNode = element.firstChild(); !childNode.isNull(); childNode = childNode.nextSibling() )
    {
        QDomElement childElement = childNode.toElement();
        if( childElement.isNull() ) continue;

        QString tagName( childElement.tagName() );
        if( tagName == Base::Xml::Record ) file_ = XmlFileRecord( childElement );

        // point lists
        else if( tagName == Xml::RefPoints ) refPoints_ = RefPoint::List( childElement );
        else if( tagName == Xml::DataSet ) _addDataSet( RefPoint::List( childElement ) );
        else Debug::Throw(0) << "XmlFileRecord::XmlFileRecord - unrecognized child " << childElement.tagName() << endl;
    }

    // make sure there is at least one data set
    if( dataSets_.empty() )
    { dataSets_ << RefPoint::List(); }
}

//_______________________________________________
QDomElement ImageRecord::domElement( QDomDocument& parent ) const
{

    Debug::Throw( "ImageRecord::domElement.\n" );

    // make sure dataSetIndex points to a valid data set
    int localIndex( dataSetIndex_ );
    if( localIndex < 0 || localIndex >= (int)dataSets_.size() || dataSets_[localIndex].isEmpty() )
    { localIndex = 0; }

    QDomElement out( parent.createElement( Xml::ImageRecord ) );
    out.setAttribute( Xml::Index, QString::number( localIndex ) );
    out.setAttribute( Xml::Flags, QString::number( flags_ ) );
    if( version_ > 0 ) out.setAttribute( Xml::ImageVersion, QString::number( version_ ) );

    out.appendChild( XmlFileRecord( file_ ).domElement( parent ) );
    out.appendChild( refPoints_.domElement( Xml::RefPoints, parent ) );

    for( const auto& dataSet:dataSets_ )
    {
        if( !dataSet.isEmpty() )
        { out.appendChild( dataSet.domElement( Xml::DataSet, parent ) ); }
    }

    return out;

}

//_______________________________________________
const ImageRecord::List ImageRecord::Helper::list( const QDomElement& element )
{

    List out;

    // parse children elements
    for(QDomNode childNode = element.firstChild(); !childNode.isNull(); childNode = childNode.nextSibling() )
    {
        QDomElement element = childNode.toElement();
        if( element.isNull() ) continue;

        // special options
        if( element.tagName() == Xml::ImageRecord )
        {

            ImageRecord record( element );
            if( !record.file().file().isEmpty() ) out.append( record );

        }

    }

    return out;

}

//_______________________________________________
const QDomElement ImageRecord::Helper::domElement( const ImageRecord::List& list, QDomDocument& parent )
{

    Debug::Throw( "ImageRecord::List::domElement.\n" );

    // create main element
    QDomElement out( parent.createElement( Xml::ImageRecordList ) );

    // loop over records
    for( const auto& record:list )
    {

        Debug::Throw() << "ImageRecord::List::domElement - " << record.file();
        if( record.file().file().isEmpty() )
        {
            Debug::Throw(0, "ImageRecord::List::domElement - attend to write empty record. Discarded.\n" );
            continue;
        }

        out.appendChild( record.domElement( parent ) );
    }

    return out;

}
