#ifndef ImageRecord_h
#define ImageRecord_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "FileRecord.h"
#include "RefPoint.h"

#include <QDomElement>
#include <QDomDocument>
#include <QList>

class ImageRecord final: private Base::Counter<ImageRecord>
{

    public:

    //* list iterator
    using ListIterator = QListIterator<ImageRecord>;

    //* constructor
    explicit ImageRecord( const File& = File(), int = 0 );

    //* constructor
    explicit ImageRecord( const QDomElement& );

    //* write to dom
    QDomElement domElement( QDomDocument& ) const;

    //*@name accessors
    //@{

    //* file
    const FileRecord& file() const
    { return file_; }

    //* file
    FileRecord& file()
    { return file_; }

    //* version
    int version() const
    { return version_; }

    //* reference points
    const RefPoint::List& refPoints() const
    { return refPoints_; }

    //* collection
    const RefPoint::Collection& collection() const
    { return dataSets_; }

    //* data set index
    int dataSetIndex() const
    { return dataSetIndex_; }

    //* flags
    enum Flag
    {
        LogX = 1<<0,
        LogY = 1<<1
    };

    Q_DECLARE_FLAGS( Flags, Flag )

    //* flags
    Flags flags() const
    { return flags_; }

    //* flags
    bool hasFlag( Flag flag ) const
    { return flags_ & flag; }

    //@}

    //*@name modifiers
    //@{

    //* file
    void setFile( const File& file )
    { file_ = FileRecord( file ); }

    //* file
    void setFile( const FileRecord& record )
    { file_ = record; }

    //* version
    void setVersion( int value )
    {  version_ = value; }

    //* reference points
    void setRefPoints( const RefPoint::List& points )
    { refPoints_ = points; }

    //* set collection
    void setCollection( const RefPoint::Collection& value )
    { dataSets_ = value; }

    //* data set index
    void setDataSetIndex( int value )
    { dataSetIndex_ = value; }

    //* flags
    void setFlags( Flags value )
    { flags_ = value; }

    //* flags
    void setFlag( Flag flag, bool value )
    {
        if( value ) flags_ |= flag;
        else flags_ &= ~flag;
    }

    //@}

    //* used to sort records according to files
    class FileFTor
    {

        public:

        //* predicate
        bool operator() (const ImageRecord& first, const ImageRecord& second) const
        { return first.file().file() < second.file().file(); }

    };

    //* used to retrieve FileRecord with identical filenames
    class SameFileFTor
    {

        public:

        //* constructor
        explicit SameFileFTor( const File& file = File(), int version = -1 ):
            file_( file ),
            version_( version )
        {}

        //* constructor
        explicit SameFileFTor( const ImageRecord& record ):
            file_( record.file().file() ),
            version_( record.version() )
        {}

        //* predicate
        bool operator() (const ImageRecord& record ) const
        { return record.file().file() == file_ && ( version_ < 0 || record.version() == version_ ); }

        //* predicate
        bool operator() (const ImageRecord& first, const ImageRecord& second) const
        { return first.file().file() == second.file().file() && first.version() == second.version(); }

        private:

        //* filename
        File file_;

        //* version
        int version_;

    };

    //* used to retrieve FileRecord with identical filenames
    class FirstOpenFTor
    {

        public:

        //* predicate
        bool operator() (const ImageRecord& first, const ImageRecord& second) const
        { return first.file().time() < second.file().time(); }

    };

    //* used to remove non existing files
    class InvalidFTor
    {
        public:

        //* predicate
        bool operator()( const ImageRecord& record )
        { return !record.file().isValid(); }

    };


    //* shortcut to list of records
    using List = QList<ImageRecord>;

    //* list helper
    class Helper
    {
        public:

        //* constructor
        static const List list( const QDomElement& );

        //* write to dom
        static const QDomElement domElement( const List&, QDomDocument& );

    };

    protected:

    //* add dataSet
    void _addDataSet( const RefPoint::List& points )
    { dataSets_ << points; }

    private:

    //* file record
    FileRecord file_;

    //* version
    int version_ = 0;

    //* reference point
    RefPoint::List refPoints_;

    //* reference points
    RefPoint::Collection dataSets_;

    //* data set index
    int dataSetIndex_ = 0;

    //* flags
    Flags flags_ = 0;

};

//* equal to operator
inline bool operator == (const ImageRecord& first, const ImageRecord& second)
{
    return
        first.file() == second.file() &&
        first.version() == second.version();
}

//* less than operator
inline bool operator < (const ImageRecord& first, const ImageRecord& second)
{
    if( first.file() == second.file() ) return first.version() < second.version();
    else return first.file() < second.file();
}

Q_DECLARE_OPERATORS_FOR_FLAGS( ImageRecord::Flags )

#endif
