/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ImageRecordConfiguration.h"

#include "BaseIconNames.h"
#include "ContextMenu.h"
#include "CppUtil.h"
#include "Debug.h"
#include "FileDialog.h"
#include "GridLayout.h"
#include "IconEngine.h"
#include "ImageRecordList.h"
#include "OptionCheckBox.h"
#include "OptionSpinBox.h"
#include "QuestionDialog.h"
#include "TreeView.h"
#include "XmlOptions.h"

#include <QGroupBox>
#include <QLabel>
#include <QLayout>

//___________________________________________
ImageRecordConfiguration::ImageRecordConfiguration( QWidget* parent, ImageRecordList& recentFiles ):
    QWidget( parent ),
    OptionWidgetList( this ),
    Counter( "ImageRecordConfiguration" ),
    recentFiles_( &recentFiles )
{
    Debug::Throw( "ImageRecordConfiguration::ImageRecordConfiguration.\n" );

    QVBoxLayout* layout = new QVBoxLayout;
    setLayout( layout );
    layout->setMargin(0);
    layout->setSpacing( 5 );

    // group box for additional options
    QGroupBox* box;
    layout->addWidget( box = new QGroupBox( tr( "Options" ), this ) );

    QVBoxLayout* vLayout = new QVBoxLayout;
    vLayout->setMargin(5);
    vLayout->setSpacing(5);
    box->setLayout( vLayout );

    QHBoxLayout* hLayout = new QHBoxLayout;
    hLayout->setSpacing(5);
    hLayout->setMargin(0);
    vLayout->addLayout( hLayout );

    // previous file history size
    OptionSpinBox *spinbox;
    hLayout->addWidget( new QLabel( tr( "Recent files history size:" ), box ) );
    hLayout->addWidget( spinbox = new OptionSpinBox( box, "DB_SIZE" ) );
    hLayout->addStretch(1);
    spinbox->setSpecialValueText( tr( " Unlimited" ) );
    spinbox->setMinimum( 0 );
    spinbox->setMaximum( 100 );
    addOptionWidget( spinbox );
    spinbox->setToolTip( tr( "Number of previously opened files to appear in the Open Previous menu" ) );

    // sort previous files by date
    OptionCheckBox* checkbox;
    vLayout->addWidget( checkbox = new OptionCheckBox( tr( "Sort recent files by date" ), box, "SORT_FILES_BY_DATE" ) );
    checkbox->setToolTip( tr( "Sort files by date rather than name in Open Previous menu" ) );
    addOptionWidget( checkbox );

    // group box for file list
    layout->addWidget( box = new QGroupBox( tr( "Files" ), this ), 1 );
    box->setLayout( new QHBoxLayout );
    box->layout()->setSpacing(5);
    box->layout()->setMargin(5);

    box->layout()->addWidget( list_ = new TreeView( box ) );
    list_->setModel( &model_ );
    list_->sortByColumn( XmlOptions::get().get<bool>( "SORT_FILES_BY_DATE" ) ? ImageRecordModel::Time : ImageRecordModel::Filename );
    list_->setSelectionMode( QAbstractItemView::ContiguousSelection );
    list_->setOptionName( "RECENTFILES_CONFIGURATION_LIST" );

    connect( list_->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), SLOT(_updateButtons()) );

    vLayout = new QVBoxLayout;
    vLayout->setSpacing(5);
    vLayout->setMargin(0);
    box->layout()->addItem( vLayout );

    // context menu
    QMenu* menu( new ContextMenu( list_ ) );

    // clean
    vLayout->addWidget( cleanButton_ = new QPushButton( tr( "Clean" ), box ) );
    cleanButton_->setToolTip( tr( "Remove invalid files" ) );
    cleanButton_->setIcon( IconEngine::get( IconNames::Delete ) );
    connect( cleanButton_, SIGNAL(clicked()), SLOT(_clean()) );

    addAction( cleanAction_ = new QAction( IconEngine::get( IconNames::Delete ), tr( "Clean" ), this ) );
    connect( cleanAction_, SIGNAL(triggered()), SLOT(_clean()) );
    menu->addAction( cleanAction_ );

    // change file
    vLayout->addWidget( changeFileButton_ = new QPushButton( tr( "Change File" ), box ) );
    changeFileButton_->setIcon( IconEngine::get( IconNames::Edit ) );
    changeFileButton_->setToolTip( tr( "Change file associated to current data set" ) );
    connect( changeFileButton_, SIGNAL(clicked()), SLOT(_changeFile()) );

    addAction( changeFileAction_ = new QAction( IconEngine::get( IconNames::Remove ), tr( "Change File" ), this ) );
    connect( changeFileAction_, SIGNAL(triggered()), SLOT(_changeFile()) );
    menu->addAction( changeFileAction_ );
    list_->addAction( changeFileAction_ );

    // remove
    vLayout->addWidget( removeButton_ = new QPushButton( tr( "Remove" ), box ) );
    removeButton_->setIcon( IconEngine::get( IconNames::Remove ) );
    removeButton_->setToolTip( tr( "Remove selected files" ) );
    connect( removeButton_, SIGNAL(clicked()), SLOT(_remove()) );

    addAction( removeAction_ = new QAction( IconEngine::get( IconNames::Remove ), tr( "Remove" ), this ) );
    connect( removeAction_, SIGNAL(triggered()), SLOT(_remove()) );
    removeAction_->setShortcut( QKeySequence::Delete );
    menu->addAction( removeAction_ );
    list_->addAction( removeAction_ );

    // reload
    vLayout->addWidget( reloadButton_ = new QPushButton( tr( "Reload" ), box ) );
    reloadButton_->setToolTip( tr( "Reload file list" ) );
    reloadButton_->setIcon( IconEngine::get( IconNames::Reload ) );
    connect( reloadButton_, SIGNAL(clicked()), SLOT(reload()) );

    addAction( reloadAction_ = new QAction( IconEngine::get( IconNames::Reload ), tr( "Reload" ), this ) );
    connect( reloadAction_, SIGNAL(triggered()), SLOT(reload()) );
    menu->addAction( reloadAction_ );

    vLayout->addStretch( 1 );

    // connections
    reloadButton_->setEnabled( false );

}

//__________________________________________________________________________
void ImageRecordConfiguration::read()
{
    Debug::Throw( "ImageRecordConfiguration::read.\n" );
    reload();
    list_->resizeColumns();
}

//__________________________________________________________________________
void ImageRecordConfiguration::write()
{
    Debug::Throw( "ImageRecordConfiguration::write.\n" );
    recentFiles_->set( ImageRecord::List( model_.get() ) );
}

//__________________________________________________________________________
void ImageRecordConfiguration::reload()
{

    Debug::Throw( "ImageRecordConfiguration::reload.\n" );
    ImageRecord::List records( recentFiles_->records() );
    model_.set( records );
    list_->selectionModel()->clear();

    reloadButton_->setEnabled( false );
    cleanButton_->setEnabled( std::find_if( records.begin(), records.end(), ImageRecord::InvalidFTor() ) != records.end() );
    _updateButtons();

}

//__________________________________________________________________________
void ImageRecordConfiguration::_updateButtons()
{

    Debug::Throw( "ImageRecordConfiguration::_updateButtons.\n" );
    QList<QModelIndex> selection( list_->selectionModel()->selectedRows() );
    removeButton_->setEnabled( !selection.empty() );
    changeFileButton_->setEnabled( selection.size() == 1 );

}

//__________________________________________________________________________
void ImageRecordConfiguration::_remove()
{

    Debug::Throw( "ImageRecordConfiguration::_remove.\n" );
    QList<QModelIndex> selection( list_->selectionModel()->selectedRows() );
    if( selection.isEmpty() ) return;

    // ask confirmation
    if( !QuestionDialog( this, tr( "Remove selected files from list ?" ) ).centerOnParent().exec() ) return;

    model_.remove( model_.get( selection ) );
    list_->selectionModel()->clear();

    reloadButton_->setEnabled( true );

    // records
    ImageRecordModel::List records( model_.get() );
    cleanButton_->setEnabled( std::find_if( records.begin(), records.end(), ImageRecord::InvalidFTor() ) != records.end() );

}

//__________________________________________________________________________
void ImageRecordConfiguration::_clean()
{
    Debug::Throw( "ImageRecordConfiguration::_clean.\n" );

    // remove invalid files
    ImageRecordModel::List records( model_.get() );
    records.erase(
        std::remove_if( records.begin(), records.end(), ImageRecord::InvalidFTor() ),
        records.end() );

    model_.set( records );

    reloadButton_->setEnabled( true );
    cleanButton_->setEnabled( false );
    _updateButtons();

}

//__________________________________________________________________________
void ImageRecordConfiguration::_changeFile()
{

    Debug::Throw( "ImageRecordConfiguration::_changeFile.\n" );
    QList<QModelIndex> selection( list_->selectionModel()->selectedRows() );
    if( selection.size() != 1 ) return;

    // get index, and image record
    QModelIndex sourceIndex( selection.front() );
    ImageRecord record( model_.get( sourceIndex ) );

    FileDialog dialog( this );
    dialog.selectFile( record.file().file() );

    auto file( dialog.getFile() );
    if( !file.isEmpty() && record.file().file() != file ) record.setFile( file );
    model_.replace( sourceIndex, record );

}
