/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ImageRecordList.h"

#include "CppUtil.h"
#include "Debug.h"
#include "File.h"
#include "Singleton.h"
#include "XmlDef.h"
#include "XmlDocument.h"
#include "XmlOptions.h"

#include <QApplication>
#include <algorithm>

//_______________________________________________
ImageRecordList::ImageRecordList( QObject* parent ):
    QObject( parent ),
    Counter( "ImageRecordList" ),
    thread_( this )
{
    connect( Base::Singleton::get().application(), SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    connect( qApp, SIGNAL(aboutToQuit()), SLOT(_saveConfiguration()) );
    _updateConfiguration();

    // thread configuration
    thread_.setCheckDuplicates( false );
    connect( &thread_, SIGNAL(recordsAvailable(const FileRecord::List&,bool)), SLOT(_processRecords(const FileRecord::List&,bool)) );

}

//_______________________________________________
bool ImageRecordList::read( File file )
{

    Debug::Throw( "ImageRecordList::read.\n" );
    if( file.isEmpty() ) file = dbFile_;
    if( file.isEmpty() )
    {

        Debug::Throw() << "ImageRecordList::read - invalid file" << endl;
        return false;

    }

    Debug::Throw() << "ImageRecordList::read - file: " << file << endl;

    // parse the file
    XmlDocument document;
    {
        QFile qfile( file );
        if( !document.setContent( &qfile ) )
        {
            Debug::Throw() << document.error() << endl;
            return false;
        }
    }

    // look for relevant element
    QDomNodeList topNodes = document.elementsByTagName( Xml::ImageRecordList );
    if( topNodes.isEmpty() )
    {
        Debug::Throw() <<  "ImageRecordList::read - top node not found" << endl;
        return false;

    }

    for(QDomNode node = topNodes.at(0).firstChild(); !node.isNull(); node = node.nextSibling() )
    {
        QDomElement element = node.toElement();
        if( element.isNull() ) continue;

        // special options
        if( element.tagName() == Xml::ImageRecord )
        {

            ImageRecord record( element );
            if( !record.file().file().isEmpty() ) _add( record, true, false );
            else Debug::Throw( "ImageRecordList::read - attend to add empty record. Discarded.\n" );

        } else Debug::Throw() << "ImageRecordList::read - unrecognized tag " << element.tagName() << endl;
    }

    Debug::Throw() <<  "ImageRecordList::read - size: " << size() << endl;
    emit contentsChanged();

    return true;
}

//_______________________________________________
bool ImageRecordList::write( File file )
{
    Debug::Throw() << "ImageRecordList::write - "
        << " records: " << records_.size()
        << endl;

    if( file.isEmpty() ) file = dbFile_;
    if( dbFile_.isEmpty() ) return false;

    // create document and read
    XmlDocument document;
    {
        QFile qtfile( file );
        document.setContent( &qtfile );
    }

    // append truncated list
    auto top = ImageRecord::Helper::domElement( _truncatedList( records_ ), document.get() );
    document.replaceChild( top );
    {
        QFile qfile( file );
        if( !qfile.open( QIODevice::WriteOnly ) ) return false;
        qfile.write( document.toByteArray() );
    }

    return true;
}

//_______________________________________________
bool ImageRecordList::_setDBFile( const File& file )
{
    Debug::Throw() << "ImageRecordList::_setDBFile - file: " << file << endl;

    // check file
    if( dbFile_ == file && !records_.empty() ) return false;

    // store file and read
    dbFile_ = file;
    read();

    // make sure file is hidden (windows only)
    if( dbFile_.localName().startsWith('.') )
    { dbFile_.setHidden(); }

    return true;

}

//______________________________________
void ImageRecordList::_updateConfiguration()
{
    Debug::Throw( "ImageRecordList::_updateConfiguration.\n" );

    // DB file
    _setDBFile( File( XmlOptions::get().raw("RC_FILE") ) );
    _setMaxSize( XmlOptions::get().get<int>( "DB_SIZE" ) );
    return;

}

//______________________________________
void ImageRecordList::_saveConfiguration()
{
    Debug::Throw( "ImageRecordList::_saveConfiguration.\n" );
    write();
}

//_______________________________________________
bool ImageRecordList::contains( const File& file, int version ) const
{ return std::find_if( records_.begin(), records_.end(), ImageRecord::SameFileFTor( file, version ) ) != records_.end(); }

//_______________________________________________
void ImageRecordList::remove( const File& file, int version )
{
    Debug::Throw() << "ImageRecordList::remove - " << file << endl;
    records_.erase(std::remove_if( records_.begin(), records_.end(), ImageRecord::SameFileFTor( file, version ) ), records_.end() );
    return;
}

//___________________________________________________
void ImageRecordList::set( const ImageRecord::List& records )
{

    Debug::Throw( "ImageRecordList::set.\n" );
    records_ = records;
    emit contentsChanged();

}

//___________________________________________________
QList<File> ImageRecordList::files() const
{
    Debug::Throw( "ImageRecordList::files.\n" );

    QList<File> out;
    for( const auto& record:_truncatedList( records_ ) )
    { out << record.file().file(); }

    return out;
}

//_______________________________________________
ImageRecord ImageRecordList::lastValidFile()
{

    Debug::Throw( "ImageRecordList::lastValidFile.\n" );

    // sort list
    std::sort( records_.begin(), records_.end(), ImageRecord::FirstOpenFTor() );
    ImageRecord::ListIterator iter( records_ );
    iter.toBack();
    while( iter.hasPrevious() )
    {
        const ImageRecord& record( iter.previous() );
        if( !check_ || record.file().isValid() ) return record;
    }

    return ImageRecord( File("") );

}

//_______________________________________________
void ImageRecordList::checkValidFiles()
{
    Debug::Throw( "ImageRecordList::checkValidFiles.\n" );
    if( !check() ) return;
    if( thread_.isRunning() ) return;

    // not very optimal, but ...
    thread_.setRecords( _fileRecords().toList() );
    thread_.start();
}

//_______________________________________________
void ImageRecordList::_processRecords( const FileRecord::List& records, bool hasInvalidRecords )
{

    Debug::Throw( "ImageRecordList::_processRecords.\n" );

    // set file records validity
    for( auto& record:records_ )
    {

        auto found = std::find_if( records.begin(), records.end(), FileRecord::SameFileFTor( record.file().file() ) );
        if( found == records.end() ) continue;

        record.file().setValid( found->isValid() );
    }

    _setCleanEnabled( hasInvalidRecords );
    emit validFilesChecked();

}

//___________________________________________________
void ImageRecordList::clean()
{
    Debug::Throw( "ImageRecordList::clean" );

    if( !check() )
    {
        records_.clear();
        return;
    }

    // remove invalid files
    records_.erase(
        std::remove_if( records_.begin(), records_.end(), ImageRecord::InvalidFTor() ),
        records_.end() );

    return;
}

//___________________________________________________
void ImageRecordList::clear()
{
    Debug::Throw( "ImageRecordList::clear" );
    records_.clear();
    return;
}
//_______________________________________________
void ImageRecordList::_setMaxSize( int value )
{

    Debug::Throw( "ImageRecordList::_setMaxSize.\n" );
    maxSize_ = value;

    return;

}

//_______________________________________________
ImageRecord& ImageRecordList::_add(
    const ImageRecord& record,
    bool update,
    bool emitSignal )
{

    // do not add empty files
    Q_ASSERT( !record.file().file().isEmpty() );

    Debug::Throw() << "ImageRecordList::_add -"
        << " file: " << record.file().file()
        << " version " << record.version()
        << " update: " << update
        << endl;

    auto iter = std::find_if( records_.begin(), records_.end(), ImageRecord::SameFileFTor( record.file().file(), record.version() ) );
    if( iter != records_.end() )
    {

        if( update )
        {
            bool changed( false );
            if( iter->flags() != record.flags() )
            {
                changed = true;
                iter->setFlags( record.flags() );
            }

            if( iter->refPoints() != record.refPoints() )
            { changed = true; }

            if( iter->collection() != record.collection() )
            { changed = true; }

            if( iter->dataSetIndex() != record.dataSetIndex() )
            { changed = true; }

            // update none the less, because equal-to operator is incomplete
            iter->setRefPoints( record.refPoints() );
            iter->setCollection( record.collection() );
            iter->setDataSetIndex( record.dataSetIndex() );

            if( iter->file().time() != record.file().time() )
            {
                iter->file().setTime( qMax( iter->file().time(), record.file().time() ) );
                changed = true;
            }

            if( changed && emitSignal ) emit contentsChanged();
        }

        return *iter;

    } else {

        Debug::Throw() << "ImageRecordList::_add - record: " << record.file() << endl;

        records_ << record;

        if( emitSignal ) emit contentsChanged();
        return records_.back();

    }

}

//___________________________________________________________________
ImageRecord::List ImageRecordList::_truncatedList( ImageRecord::List records ) const
{
    // shrink list
    if( maxSize_ > 0 && int(records.size()) > maxSize_ )
    {

        std::sort( records.begin(), records.end(), ImageRecord::FirstOpenFTor() );
        if( check_ )
        {

            while( int(records.size()) > maxSize_ )
            {
                auto iter( std::find_if( records.begin(), records.end(), ImageRecord::InvalidFTor() ) );
                if( iter != records.end() ) records.erase( iter );
                else break;
            }

        }

        // remove oldest files
        while( int(records.size()) > maxSize_ )
        { records.removeFirst(); }

    }

    return records;
}
