#ifndef ImageRecordList_h
#define ImageRecordList_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "Debug.h"
#include "File.h"
#include "ImageRecord.h"
#include "TimeStamp.h"
#include "ValidFileThread.h"

#include <QObject>
#include <QList>

//* handles list of files saved into resource file for later reopening
class ImageRecordList: public QObject, private Base::Counter<ImageRecordList>
{

    //* Qt meta object declaration
    Q_OBJECT

    public:

    //* constructor
    explicit ImageRecordList( QObject* );

    //* returns true if file is found in list
    bool contains( const ImageRecord& record ) const
    { return contains( record.file().file(), record.version() ); }

    //* returns true if file is found in list
    bool contains( const File&, int = -1 ) const;

    //* add file.
    ImageRecord& add( const File& file )
    { return _add( ImageRecord( file ), false ); }

    //* add file.
    ImageRecord& add( const ImageRecord& record )
    { return _add( record ); }

    //* remove file from database
    void remove( const File&, int = -1 );

    //* get filerecord associated to a name
    /** creates new fileRecord if not found */
    ImageRecord& get( const File& file, int version = 0 )
    { return _add( ImageRecord( file, version ), false ); }

    //* gets file list size
    int size() const
    { return records_.size(); }

    //* all records
    ImageRecord::List records() const
    { return _truncatedList( records_ ); }

    //* set record
    void set( const ImageRecord::List& );

    //* all files
    QList< File > files() const;

    //* get last valid file
    ImageRecord lastValidFile();

    //* returns true if file list can be cleaned
    bool cleanEnabled() const
    { return (check()) ? cleanEnabled_ : !records_.empty(); }

    //* clean files. Remove either invalid or all files, depending on check_
    void clean();

    //* clear files. Remove all
    void clear();

    //* check flag
    bool check() const
    { return check_; }

    //* check_ flag
    void setCheck( bool value )
    { check_ = value; }

    //* Read fileList from file
    bool read( File = File() );

    //* write fileList to file
    bool write( File = File() );

    Q_SIGNALS:

    //* emitted when thread has completed validity check
    void validFilesChecked();

    //* emitted when contents is changed
    void contentsChanged();

    public Q_SLOTS:

    //* run thread to check file validity
    void checkValidFiles();

    protected:

    //* set db file
    bool _setDBFile( const File& file );

    //* maximum Size
    void _setMaxSize( int value );

    //* maximum size
    int _maxSize() const
    { return maxSize_; }

    //* add record to current list
    ImageRecord& _add(
        const ImageRecord& record,
        bool update_timestamp = true,
        bool emit_signal = true );

    //* truncate list if larger than maxSize_
    ImageRecord::List _truncatedList( ImageRecord::List ) const;

    //* list of files records
    FileRecord::Set _fileRecords() const
    {
        FileRecord::Set out;
        for( const auto& record:records_ )
        { out << record.file(); }
        return out;
    }

    protected Q_SLOTS:

    //* process records from threads
    void _processRecords( const FileRecord::List&, bool );

    private Q_SLOTS:

    //* update configuration
    void _updateConfiguration();

    //* save configuration
    void _saveConfiguration();

    private:

    //* clean enabled
    void _setCleanEnabled( bool value )
    { cleanEnabled_ = value; }

    //* maximum size (negative means no limit)
    int maxSize_ = -1;

    //* if true, check file validity
    bool check_ = true;

    //* true if clean action is enabled
    bool cleanEnabled_ = false;

    //* thread to check file validity
    ValidFileThread thread_;

    //* current list of files
    ImageRecord::List records_;

    //* file from/to wich the files are saved
    File dbFile_;

};
#endif
