/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ImageRecordModel.h"

#include "CppUtil.h"
#include "XmlOptions.h"

#include <QPalette>
#include <algorithm>

//__________________________________________________________________
ImageRecordModel::ImageRecordModel( QObject* parent ):
    ListModel<ImageRecord>( parent ),
    Counter( "ImageRecordModel" )
{
    Debug::Throw("ImageRecordModel::ImageRecordModel.\n" );

    columnTitles_
        << tr( "File" )
        << tr( "Path" )
        << tr( "Last Accessed" );

}

//__________________________________________________________________
int ImageRecordModel::columnCount( const QModelIndex& ) const
{ return columnTitles_.size(); }

//__________________________________________________________________
int ImageRecordModel::findColumn( const QString& value ) const
{
    for( int index = 0; index < columnTitles_.size(); index++ )
    { if( columnTitles_[index] == value ) return index;  }
    return -1;
}

//__________________________________________________________________
Qt::ItemFlags ImageRecordModel::flags(const QModelIndex &index) const
{

    // default flags
    Qt::ItemFlags flags;
    if( !index.isValid() ) return flags;

    // default flags
    flags |=  Qt::ItemIsEnabled |  Qt::ItemIsSelectable;

    return flags;

}

//__________________________________________________________________
QVariant ImageRecordModel::data( const QModelIndex& index, int role ) const
{

    // check index
    if( !contains( index ) ) return QVariant();

    // retrieve associated file info
    const ImageRecord& record( get(index) );

    // return text associated to file and column
    if( role == Qt::DisplayRole ) {

        switch( index.column() )
        {

            case Filename:
            {
                // store local nmae
                QString localName( record.file().file().localName() );
                QString buffer = localName.isEmpty() ? tr( "untitled" ):localName;
                if( record.version() > 0 ) buffer += QString( " (v%1)" ).arg( record.version()+1 );
                return buffer;
            }

            case Path: return record.file().file().path().get();
            case Time: return TimeStamp( record.file().time() ).toString();
            default: return QVariant();

        }

    } else if( role == Qt::ForegroundRole ) {

        if( !record.file().isValid() ) return QPalette().color( QPalette::Disabled, QPalette::Text );
        else return QVariant();

    } else if( role == Qt::ToolTipRole ) return record.file().file().get();

    return QVariant();

}

//__________________________________________________________________
QVariant ImageRecordModel::headerData(int section, Qt::Orientation orientation, int role) const
{

    if(
        orientation == Qt::Horizontal &&
        role == Qt::DisplayRole &&
        section >= 0 &&
        section < (int) columnTitles_.size() )
    { return columnTitles_[section]; }

    // return empty
    return QVariant();

}

//____________________________________________________________
void ImageRecordModel::_sort( int column, Qt::SortOrder order )
{ std::sort( _get().begin(), _get().end(), SortFTor( column, order, columnTitles_ ) ); }

//________________________________________________________
bool ImageRecordModel::SortFTor::operator () ( ImageRecord first, ImageRecord second ) const
{

    if( order_ == Qt::DescendingOrder ) std::swap( first, second );

    switch( type_ )
    {

        case Filename: return first.file().file().localName() < second.file().file().localName();
        case Path: return first.file().file().path() < second.file().file().path();
        case Time: return (first.file().time() != second.file().time() ) ? (first.file().time() < second.file().time()):first.file().file().localName() < second.file().file().localName();
        default: return false;
    }

}
