#ifndef MainWindow_h
#define MainWindow_h
/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "BaseMainWindow.h"
#include "Counter.h"
#include "ImageRecord.h"

#include <QBasicTimer>
#include <QTimerEvent>
#include <QLabel>
#include <QAction>
#include <QScrollArea>
#include <QUndoStack>

// forward declaration
class Menu;
class RefPointPanel;
class DisplayWidget;
class NotificationWidget;
class RefPointListWidget;
class BaseStatusBar;

class MainWindow: public BaseMainWindow, private Base::Counter<MainWindow>
{

    Q_OBJECT

    public:

    //* constructor
    explicit MainWindow( QWidget* = nullptr );

    //* image record
    void setImageRecord( const ImageRecord& );

    //* image record
    bool hasImageRecord() const;

    //* image record
    ImageRecord imageRecord();

    //* display
    DisplayWidget& displayWidget() const
    { return *displayWidget_; }

    //* reference point panel
    RefPointPanel& refPointPanel() const
    { return *refPointPanel_; }

    //* list widget
    RefPointListWidget& refPointListWidget() const
    { return *refPointListWidget_; }

    //* clear
    void clear();

    //*@name actions
    //@{

    //* default errors
    QAction& defaultErrorsConfigurationAction() const
    { return *defaultErrorsConfigurationAction_; }

    //* default errors
    QAction& defaultErrorsAction() const
    { return *defaultErrorsAction_; }

    //* default systematic errors
    QAction& defaultErrorsSystAction() const
    { return *defaultErrorsSystAction_; }

    //* undo action
    QAction& undoAction() const
    { return *undoAction_; }

    //* redo action
    QAction& redoAction() const
    { return *redoAction_; }

    //@}

    //*@name undo/redo stack
    //@{

    //* add point
    void updateReferencePoint( int, const RefPoint& );

    //* remove point
    void removeReferencePoint( int );

    //* add point
    void addPoint( int, const RefPoint& );

    //* add point
    void insertPoint( int, int, const RefPoint& );

    //* remove point
    void removePoint( int, const RefPoint& );

    //* replace point
    void replacePoint( int, const RefPoint&, const RefPoint& );

    //@}

    protected:

    //* timer event
    void timerEvent( QTimerEvent* );

    protected Q_SLOTS:

    //* pointer position changed
    void _pointerMoved( QPoint, RefPoint::HoverMode );

    //* display widget left
    void _displayWidgetEntered();

    //* display widget left
    void _displayWidgetLeft();

    //* edit reference point
    void _editRefPoint( QAction* );

    //* edit reference point
    void _editRefPoint( int );

    //* clear reference point
    void _clearRefPoint( QAction* );

    //* clear reference point
    void _clearRefPoint( int );

    //* update ref points when changed
    void _updateRefPoints();

    //* update points
    void _updatePoints();

    //* update selected points
    void _updateSelection();

    //* update selected points
    void _updateRefPointSelection();

    //* select point
    void _selectPoint();

    //* add point
    void _addPoint( QPointF );

    //* remove point
    void _removePoint();

    //* remove points
    void _removePoints( const RefPoint::List& );

    //* move points up
    void _movePointsUp( const RefPoint::List& );

    //* move points up
    void _movePointsDown( const RefPoint::List& );

    //* copy
    void _copyPoint();

    //* paste
    void _pastePoint( const QPointF& );

    //* edit errors
    void _editErrors();

    //* configure default errors
    void _configureDefaultErrors();

    //* edit errors
    void _addDefaultErrors();

    //* edit errors
    void _addDefaultErrorsSyst();

    //* edit errors
    void _editErrors( RefPoint );

    //* edit errors
    void _addDefaultErrors( RefPoint );

    //* edit errors
    void _addDefaultErrorsSyst( RefPoint );

    //* edit errors
    void _addDefaultErrorsToAll();

    //* edit errors
    void _addDefaultErrorsSystToAll();

    //* drag point
    void _dragPoint( const QPointF&, RefPoint::DragMode );

    //* move point(s) using arrow keys
    void _movePointsUp() { return _movePoints( Qt::Key_Up ); }
    void _movePointsDown() { return _movePoints( Qt::Key_Down ); }
    void _movePointsLeft() { return _movePoints( Qt::Key_Left ); }
    void _movePointsRight() { return _movePoints( Qt::Key_Right ); }

    //* regularize points
    void _regularize();

    protected:

    //* notification widget
    NotificationWidget& _notificationWidget() const
    { return *notificationWidget_; }

    //* move point(s) using arrow keys
    void _movePoints( Qt::Key );

    private Q_SLOTS:

    //* splitter moved
    void _splitterMoved();

    //* configuration
    void _updateConfiguration();

    private:

    //* install actions
    void _installActions();

    //* image record
    ImageRecord record_;

    //* default errors
    bool defaultErrorsFromOptions_ = true;
    RefPoint::Error defaultErrorX_;
    RefPoint::Error defaultErrorY_;
    RefPoint::Error defaultErrorSystX_;
    RefPoint::Error defaultErrorSystY_;

    //* menu
    Menu* menu_;

    //* local widget to allow size adjustment
    class Widget: public QWidget, private Base::Counter<Widget>
    {

        public:

        //* constructor
        explicit Widget( QWidget* parent = nullptr ):
            QWidget( parent ),
            Counter( "MainWindow::Counter" )
        {}

        //* default size
        void setDefaultWidth( int );

        //* default size
        void setDefaultHeight( int );

        //* default width
        int defaultWidth() const
        { return defaultWidth_; }

        //* default width
        int defaultHeight() const
        { return defaultHeight_; }

        //* size
        QSize sizeHint() const;

        private:

        //* default width;
        int defaultWidth_ = 0;

        //* default height
        int defaultHeight_ = 0;

    };

    //* containers
    Widget* listContainer_ = nullptr;

    //* scroll area
    QScrollArea* scrollArea_ = nullptr;

    //* ref point panel
    RefPointPanel* refPointPanel_ = nullptr;

    //* display widget
    DisplayWidget* displayWidget_ = nullptr;

    //* notification widget
    NotificationWidget* notificationWidget_ = nullptr;

    //* ref point list
    RefPointListWidget* refPointListWidget_ = nullptr;

    //* file display lineEdit
    QLabel* fileEditor_ = nullptr;

    //* status bar
    BaseStatusBar* statusbar_ = nullptr;

    //* undo stack
    QUndoStack* undoStack_ = nullptr;

    //*@name actions
    //@{

    //* configure default errors
    QAction* defaultErrorsConfigurationAction_ = nullptr;

    //* default errors action
    QAction* defaultErrorsAction_ = nullptr;

    //* default errors action
    QAction* defaultErrorsSystAction_ = nullptr;

    //* undo action
    QAction* undoAction_ = nullptr;

    //* redo action
    QAction* redoAction_ = nullptr;

    //@}

    //* true if some point was hovered
    bool wasHovered_ = false;

    //* resize timer
    /** needed to store Keyword list width */
    QBasicTimer resizeTimer_;

    //* local string format
    QString localFormat_;

};

#endif
