/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "NotificationWidget.h"

#include "Debug.h"
#include "Singleton.h"
#include "XmlOptions.h"

#include <QPainter>
#include <QStyle>
#include <QStyleOptionFrame>

//_____________________________________________________________
NotificationWidget::NotificationWidget( QWidget* parent ):
    QLabel( parent ),
    Counter( "NotificationWidget" ),
    animation_( this )
{
    Debug::Throw( "NotificationWidget::NotificationWidget.\n" );

    setAttribute(Qt::WA_OpaquePaintEvent, false);
    setFocusPolicy(Qt::NoFocus);
    setAttribute(Qt::WA_TransparentForMouseEvents, true);
    setContextMenuPolicy(Qt::NoContextMenu);

    setForegroundRole( QPalette::ToolTipText );
    setBackgroundRole( QPalette::ToolTipBase );
    setMargin( 5 );

    // setup animation
    animation_.setStartValue( 0 );
    animation_.setEndValue( 255 );
    animation_.setTargetObject( this );
    animation_.setDirection( QAbstractAnimation::Forward );
    animation_.setPropertyName( "opacity" );

    // configuration
    connect( Base::Singleton::get().application(), SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    _updateConfiguration();

}

//_____________________________________________________________
void NotificationWidget::setText( const QString& text )
{

    // do nothing if disabled
    if( !enabled_ ) return;

    Debug::Throw() << "NotificationWidget::setText - \"" << text << "\"" << endl;

    if( text.isEmpty() ) hide();
    else {

        QLabel::setText( text );
        adjustSize();
        show();
        if( autoHide_ && autoHideDelay_ > 0 )
        { timer_.start( autoHideDelay_, this ); }

    }

}

//_____________________________________________________________
void NotificationWidget::setVisible( bool visible )
{
    Debug::Throw( "Notificaiton::show.\n" );
    if( visible )
    {
        if( !isVisible() ) QWidget::setVisible( true );
        if( isAnimated_ )
        {
            animation_.setDirection( QAbstractAnimation::Forward );
            if( animation_.state() != QAbstractAnimation::Running && opacity_ != 255 )
            { animation_.start(); }
        }

    } else {

        if( !isVisible() ) return;
        if( !isAnimated_ ) QWidget::setVisible( false );
        else {

            animation_.setDirection( QAbstractAnimation::Backward );
            if( animation_.state() != QAbstractAnimation::Running )
            { animation_.start(); }

        }

    }

    return;

}

//_____________________________________________________________
void NotificationWidget::setOpacity( int value )
{

    if( opacity_ == value ) return;
    opacity_ = value;

    if( opacity_ == 0 && animation_.direction() == QAbstractAnimation::Backward ) QWidget::setVisible( false );
    else update();

}

//_____________________________________________________________
void NotificationWidget::paintEvent( QPaintEvent* event )
{

    if( (!enabled_) || text().isEmpty() ) return;

    QPainter painter( this );
    painter.setClipRegion( event->region() );
    painter.setRenderHints( QPainter::Antialiasing );

    QStyleOptionFrame option;
    option.init(this);

    // alter palette
    if( opacity_ != 255 )
    {
        QColor foreground( option.palette.color( foregroundRole() ) );
        QColor background( option.palette.color( backgroundRole() ) );
        background.setAlpha( opacity_ );
        foreground.setAlpha( opacity_ );

        option.palette.setColor( foregroundRole(), foreground );
        option.palette.setColor( backgroundRole(), background );
    }

    style()->drawPrimitive( QStyle::PE_PanelTipLabel, &option, &painter, this );

    painter.end();

    QLabel::paintEvent( event );

}

//_____________________________________________________________
void NotificationWidget::timerEvent( QTimerEvent* event )
{
    if( event->timerId() == timer_.timerId() )
    {

        timer_.stop();
        hide();

    } else return QLabel::timerEvent( event );
}

//_____________________________________________________________
void NotificationWidget::_updateConfiguration()
{

    enabled_ = XmlOptions::get().get<bool>( "SHOW_NOTIFICATIONS" );
    if( !enabled_ ) hide();

    autoHide_ = XmlOptions::get().get<bool>( "AUTO_HIDE_NOTIFICATIONS" );
    autoHideDelay_ = XmlOptions::get().get<int>( "AUTO_HIDE_DELAY" );
    isAnimated_ = XmlOptions::get().get<bool>( "ANIMATE_NOTIFICATIONS" );
    animation_.setDuration( XmlOptions::get().get<int>( "ANIMATION_DURATION" ) );

    if( !isAnimated_ ) setOpacity( 255 );

}
