/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "PrintListDialog.h"

#include "IconEngine.h"
#include "IconNames.h"
#include "TextEditor.h"
#include "XmlOptions.h"

#include <QLabel>

//______________________________________________________
PrintListDialog::PrintListDialog( QWidget* parent ):
    CustomDialog( parent, CloseButton )
{

    Debug::Throw( "PrintListDialog::PrintListDialog.\n" );
    setOptionName( "PRINT_DIALOG" );

    // main grid layout
    mainLayout().addItem( gridLayout_ = new QGridLayout );
    gridLayout_->setMargin(0);
    gridLayout_->setSpacing(2);

    // format container
    formatContainer_ = new QWidget( this );
    QHBoxLayout* hLayout;
    formatContainer_->setLayout( hLayout = new QHBoxLayout );
    hLayout->setMargin(0);
    hLayout->setSpacing(0);

    QLabel* label;
    hLayout->addWidget( label = new QLabel( tr( "Format:" ), formatContainer_ ) );
    label->setAlignment( Qt::AlignRight|Qt::AlignVCenter );

    hLayout->addWidget( formatComboBox_ = new QComboBox( formatContainer_ ) );
    formatComboBox_->addItems( QStringList() << tr( "Raw" ) << tr( "LaTeX" ) << tr( "C" ) );
    formatComboBox_->setCurrentIndex( XmlOptions::get().get<int>( "PRINT_FORMAT" ) );

    hLayout->addWidget( formatToolButton_ = new QToolButton( formatContainer_ ) );
    formatToolButton_->setIcon( IconEngine::get( IconNames::Configure ) );
    formatToolButton_->setText( tr( "Configure Marker Format for Printing" ) );
    formatToolButton_->setAutoRaise( true );

    hLayout->addStretch( 1 );

    // main text editor
    editor_ = new TextEditor( this );
    editor_->setReadOnly( true );

    // container for checkboxes
    checkBoxContainer_ = new QWidget( this );
    checkBoxContainer_->setLayout( checkBoxLayout_ = new QVBoxLayout );
    checkBoxLayout_->setMargin(0);
    checkBoxLayout_->setSpacing(5);
    checkBoxLayout_->addWidget( new QLabel( tr( "Data Sets:" ), checkBoxContainer_ ) );

    connect( formatComboBox_, SIGNAL(currentIndexChanged(int)), SLOT(updateText()) );
    connect( formatComboBox_, SIGNAL(currentIndexChanged(int)), SLOT(_saveFormat()) );

}

//______________________________________________________
void PrintListDialog::setCollection( const RefPoint::Collection& collection, int index )
{
    Debug::Throw( "PrintListDialog::setCollection.\n" );

    // clear checkboxes
    for( auto&& checkBox:checkBoxes_ ) delete checkBox;
    checkBoxes_.clear();

    // store collection
    collection_ = collection;

    // clear layout
    if( gridLayout_->indexOf( formatContainer_ ) >= 0 ) gridLayout_->removeWidget( formatContainer_ );
    if( gridLayout_->indexOf( checkBoxContainer_ ) >= 0 ) gridLayout_->removeWidget( checkBoxContainer_ );
    if( gridLayout_->indexOf( editor_ ) >= 0 ) gridLayout_->removeWidget( editor_ );

    if( collection_.size() == 1 )
    {

        // setup layout
        gridLayout_->addWidget( formatContainer_, 0, 0, 1, 1 );
        gridLayout_->addWidget( editor_, 1, 0, 1, 1 );
        gridLayout_->setRowStretch( 1, 1 );

        // hide checkbox container
        checkBoxContainer_->hide();

        // clear editor and print dataSet
        editor_->clear();
        editor_->setText( _printDataSet( 0 ) );

    } else {

        gridLayout_->addWidget( formatContainer_, 0, 0, 1, 1 );
        gridLayout_->addWidget( checkBoxContainer_, 1, 0, 1, 1 );
        gridLayout_->addWidget( editor_, 0, 1, 2, 1 );
        gridLayout_->setRowStretch( 1, 1 );
        gridLayout_->setColumnStretch( 1, 1 );
        checkBoxContainer_->show();

        // install checkboxes
        for( int i = 0; i < (int) collection_.size(); i++ )
        {
            const QString buffer = QString( tr( "Set %1" ) ).arg( i+1 );
            QCheckBox* checkBox( new QCheckBox( buffer, checkBoxContainer_ ) );
            checkBoxLayout_->addWidget( checkBox );
            checkBoxes_.append( checkBox );
            checkBox->setChecked( index == i || index < 0 );
            connect( checkBox, SIGNAL(toggled(bool)), SLOT(updateText()) );
        }

        checkBoxLayout_->addStretch( 1 );


        // print
        updateText();

    }

}

//______________________________________________________
void PrintListDialog::updateText()
{

    Debug::Throw( "PrintListDialog::updateText.\n" );
    if( collection_.size() == 1 )
    {

        // clear editor and print dataSet
        editor_->clear();
        editor_->setText( _printDataSet( 0 ) );

    } else {

        // get text stream
        QString buffer;
        QTextStream out( &buffer );
        for( int index = 0; index < (int)checkBoxes_.size(); index++ )
        {
            if( !checkBoxes_[index]->isChecked() ) continue;
            out << _printDataSet( index );
        }

        // fill editor
        editor_->clear();
        editor_->setText( buffer );

    }

}

//______________________________________________________
void PrintListDialog::_saveFormat()
{ XmlOptions::get().set<int>( "PRINT_FORMAT", formatComboBox_->currentIndex() ); }

//______________________________________________________
QString PrintListDialog::_printDataSet( int index ) const
{
    Debug::Throw( "PrintListDialog::_printDataSet.\n" );

    // check index
    if( index < 0 || index >= (int)collection_.size() ) return QString();

    // get how many checkboxes are checked
    int nChecked( 0 );
    for( int i = 0; i < (int)checkBoxes_.size(); i++ )
    { if( checkBoxes_[i]->isChecked() ) nChecked++; }
    const bool single( nChecked <= 1 );

    // print
    if( collection_.size() == 1 || single ) return _printDataSet( collection_[index] );
    else
    {
        QString name( tr( "set" ) );
        QTextStream( &name ) << index+1;
        return _printDataSet( collection_[index], name );
    }
}

//______________________________________________________
QString PrintListDialog::_printDataSet( const RefPoint::List& points, const QString& name ) const
{
    switch( formatComboBox_->currentIndex() )
    {
        case FormatRaw: return _printDataSetRaw( points, name );
        case FormatLatex: return _printDataSetLatex( points, name );
        case FormatC: return _printDataSetC( points, name );
        default: return QString();
    }
}

//______________________________________________________
QString PrintListDialog::_printDataSetRaw( const RefPoint::List& points, const QString& name ) const
{
    Debug::Throw( "PrintListDialog::_printDataSetRaw.\n" );
    if( points.isEmpty() ) return QString();

    // get arrays
    const Columns columns( _columns( points ) );
    const RefPointStringList stringList = refPointStringList( points,
        XmlOptions::get().get<QString>( "LOCAL_FORMAT" ),
        XmlOptions::get().get<QString>( "ERROR_FORMAT" ) );

    QStringList xList;
    QStringList xStatList;
    QStringList xSystList;

    QStringList yList;
    QStringList yStatList;
    QStringList ySystList;

    // insert array names
    xList << _varName( "x", name );
    if( columns & ColumnsXStat ) xStatList << _varName( "x_stat", name );
    if( columns & ColumnsXSyst ) xSystList << _varName( "x_syst", name );

    yList << _varName( "y", name );
    if( columns & ColumnsYStat ) yStatList << _varName( "y_stat", name );
    if( columns & ColumnsYSyst ) ySystList << _varName( "y_syst", name );

    // arrays
    for( const auto& refPoint:stringList )
    {
        xList << refPoint.x();
        if( columns & ColumnsXStat ) xStatList << QString( "+" ) + refPoint.xStatPlus() + ", -" + refPoint.xStatMinus();
        if( columns & ColumnsXSyst ) xSystList << QString( "+" ) + refPoint.xSystPlus() + ", -" + refPoint.xSystMinus();

        yList << refPoint.y();
        if( columns & ColumnsYStat ) yStatList << QString( "+" ) + refPoint.yStatPlus() + ", -" + refPoint.yStatMinus();
        if( columns & ColumnsYSyst ) ySystList << QString( "+" ) + refPoint.ySystPlus() + ", -" + refPoint.ySystMinus();

    }

    // normalize arrays
    _normalize( xList );
    _normalize( xStatList );
    _normalize( xSystList );

    _normalize( yList );
    _normalize( yStatList );
    _normalize( ySystList );

    QString buffer;
    QTextStream out( &buffer );
    for( int index = 0; index < (int) points.get().size() + 1; index++ )
    {
        out << xList[index] << "  ";
        if( columns & ColumnsXStat ) out << xStatList[index] << "  ";
        if( columns & ColumnsXSyst ) out << xSystList[index] << "  ";

        out << " ";
        out << yList[index] << "  ";
        if( columns & ColumnsYStat ) out << yStatList[index] << "  ";
        if( columns & ColumnsYSyst ) out << ySystList[index] << "  ";
        out << endl;
    }

    out << endl;
    return buffer;

}

//______________________________________________________
QString PrintListDialog::_printDataSetLatex( const RefPoint::List& points, const QString& name ) const
{
    Debug::Throw( "PrintListDialog::_printDataSetLatex.\n" );
    if( points.isEmpty() ) return QString();

    // get arrays
    const Columns columns( _columns( points ) );
    const RefPointStringList stringList = refPointStringList( points,
        XmlOptions::get().get<QString>( "LOCAL_FORMAT" ),
        XmlOptions::get().get<QString>( "ERROR_FORMAT" ) );

    QStringList xList;
    QStringList xStatList;
    QStringList xSystList;

    QStringList yList;
    QStringList yStatList;
    QStringList ySystList;

    // insert array names
    xList << _varName( "x", name ).replace( '_', "\\_" );
    if( columns & ColumnsXStat ) xStatList << _varName( "x_stat", name ).replace( '_', "\\_" );
    if( columns & ColumnsXSyst ) xSystList << _varName( "x_syst", name ).replace( '_', "\\_" );

    yList << _varName( "y", name ).replace( '_', "\\_" );
    if( columns & ColumnsYStat ) yStatList << _varName( "y_stat", name ).replace( '_', "\\_" );
    if( columns & ColumnsYSyst ) ySystList << _varName( "y_syst", name ).replace( '_', "\\_" );

    // arrays
    for( const auto& refPoint:stringList )
    {
        xList << QString( "$" ) + refPoint.x() + "$";
        if( columns & ColumnsXStat ) xStatList << QString( "$+" ) + refPoint.xStatPlus() + "$, $-" + refPoint.xStatMinus()+"$";
        if( columns & ColumnsXSyst ) xSystList << QString( "$+" ) + refPoint.xSystPlus() + "$, $-" + refPoint.xSystMinus()+"$";

        yList << QString( "$" ) + refPoint.y() + "$";
        if( columns & ColumnsYStat ) yStatList << QString( "$+" ) + refPoint.yStatPlus() + "$, $-" + refPoint.yStatMinus()+"$";
        if( columns & ColumnsYSyst ) ySystList << QString( "$+" ) + refPoint.ySystPlus() + "$, $-" + refPoint.ySystMinus()+"$";

    }

    // normalize arrays
    _normalize( xList );
    _normalize( xStatList );
    _normalize( xSystList );

    _normalize( yList );
    _normalize( yStatList );
    _normalize( ySystList );

    QString buffer;
    QTextStream out( &buffer );

    // header
    out << "\\begin{tabular}{c|c";
    if( columns & ColumnsXStat ) out << "|c";
    if( columns & ColumnsXSyst ) out << "|c";
    if( columns & ColumnsYStat ) out << "|c";
    if( columns & ColumnsYSyst ) out << "|c";
    out << "}" << endl;

    for( int index = 0; index < (int) points.size() + 1; index++ )
    {
        out << " " << xList[index] << " ";
        if( columns & ColumnsXStat ) out << "& " << xStatList[index] << " ";
        if( columns & ColumnsXSyst ) out << "& " << xSystList[index] << " ";

        out << "& " << yList[index] << " ";
        if( columns & ColumnsYStat ) out << "& " << yStatList[index] << " ";
        if( columns & ColumnsYSyst ) out << "& " << ySystList[index] << " ";
        out << "\\\\" << endl;

        if( index == 0 ) out << "\\hline" << endl;
    }

    out << "\\end{tabular}" << endl;
    out << endl;
    return buffer;

}

//______________________________________________________
QString PrintListDialog::_printDataSetC( const RefPoint::List& points, const QString& name ) const
{
    Debug::Throw( "PrintListDialog::_printDataSetC.\n" );
    if( points.isEmpty() ) return QString();


    QString buffer;
    QTextStream out( &buffer );
    out << "    const int " << _varName( "n_points", name ) << "  = " << points.size() << ";" << endl;

    const Columns columns( _columns( points ) );
    const RefPointStringList stringList = refPointStringList( points,
        XmlOptions::get().get<QString>( "LOCAL_FORMAT" ),
        XmlOptions::get().get<QString>( "ERROR_FORMAT" ) );

    QStringList xList;
    QStringList xStatPlusList;
    QStringList xStatMinusList;
    QStringList xSystPlusList;
    QStringList xSystMinusList;

    QStringList yList;
    QStringList yStatPlusList;
    QStringList yStatMinusList;
    QStringList ySystPlusList;
    QStringList ySystMinusList;

    // arrays
    for( const auto& refPoint:stringList )
    {
        xList << refPoint.x();
        if( columns & ColumnsXStat )
        {
            xStatPlusList << refPoint.xStatPlus();
            xStatMinusList << refPoint.xStatMinus();
        }

        if( columns & ColumnsXSyst )
        {
            xSystPlusList << refPoint.xSystPlus();
            xSystMinusList << refPoint.xSystMinus();
        }

        yList << refPoint.y();
        if( columns & ColumnsYStat )
        {
            yStatPlusList << refPoint.yStatPlus();
            yStatMinusList << refPoint.yStatMinus();
        }

        if( columns & ColumnsYSyst )
        {
            ySystPlusList << refPoint.ySystPlus();
            ySystMinusList << refPoint.ySystMinus();
        }

    }

    // horizontal
    out << _printArray( _varName( "x", name ), xList );
    if( columns & ColumnsXStat )
    {
        out << _printArray( _varName( "x_err_plus", name ), xStatPlusList );
        out << _printArray( _varName( "x_err_minus", name ), xStatMinusList );
    }

    if( columns & ColumnsXSyst )
    {
        out << _printArray( _varName( "x_err_syst_plus", name ), xSystPlusList );
        out << _printArray( _varName( "x_err_syst_minus", name ), xSystMinusList );
    }

    // vertical
    out << _printArray( _varName( "y", name ), yList );
    if( columns & ColumnsYStat )
    {
        out << _printArray( _varName( "y_err_plus", name ), yStatPlusList );
        out << _printArray( _varName( "y_err_minus", name ), yStatMinusList );
    }

    if( columns & ColumnsYSyst )
    {
        out << _printArray( _varName( "y_err_syst_plus", name ), ySystPlusList );
        out << _printArray( _varName( "y_err_syst_minus", name ), ySystMinusList );
    }

    out << "" << endl;

    return buffer;

}

//______________________________________________________
QString PrintListDialog::_printArray( const QString& name, const QStringList& array ) const
{
    Debug::Throw( "PrintListDialog::_printArray.\n" );
    QString buffer;
    QTextStream out( &buffer );

    out << "    double " << name << "[" << array.size() << "] = { ";
    for( int index = 0; index < array.size(); ++index )
    {
        if( index == 0 ) out << array[index];
        else out << ", " << array[index];
    }

    out << " };" << endl;

    return buffer;
}

//______________________________________________________
PrintListDialog::Columns PrintListDialog::_columns( const RefPoint::List& points ) const
{
    Debug::Throw( "PrintListDialog::_columns.\n" );
    Columns out( ColumnsNone );
    if( points.isEmpty() ) return out;

    out |= Columns(ColumnsX|ColumnsY);
    for( const auto& refPoint:points.get() )
    {
        if( refPoint.hasErrorX() ) out |= ColumnsXStat;
        if( refPoint.hasErrorY() ) out |= ColumnsYStat;
        if( refPoint.hasErrorSystX() ) out |= ColumnsXSyst;
        if( refPoint.hasErrorSystY() ) out |= ColumnsYSyst;
        if( out == ColumnsAll ) break;
    }

    return out;
}

//______________________________________________________
int PrintListDialog::_maxLength( const QStringList& array ) const
{
    int out( -1 );
    for( const auto& refPoint:array )
    { if( out < 0 || refPoint.length() > out ) out = refPoint.length(); }
    return out;
}

//______________________________________________________
void PrintListDialog::_normalize( QStringList& array ) const
{
    const int maxLength( _maxLength( array ) );
    for( auto& line:array )
    { line = line.leftJustified( maxLength, ' ' ); }
}

//______________________________________________________
PrintListDialog::RefPointString& PrintListDialog::RefPointString::set( const RefPoint& point, const QString& localFormat, const QString& errorFormat )
{
    Debug::Throw( "PrintListDialog::RefPointString::set.\n" );

    // position
    x_ = _string( point.local().x(), localFormat );
    y_ = _string( point.local().y(), localFormat );

    // statistical errors (horizontal)
    xStatPlus_.clear();
    xStatMinus_.clear();

    {
        double plus( point.hasErrorX() ? point.errorX().plus_ : 0 );
        double minus( point.hasErrorX() ? point.errorX().minus_ : 0 );
        if( point.errorX().relative_ )
        {
            plus *= point.local().x();
            minus *= point.local().x();
        }

        xStatPlus_ = _string( plus, errorFormat );
        xStatMinus_ = _string( minus, errorFormat );
    }

    // statistical errors (vertical)
    yStatPlus_.clear();
    yStatMinus_.clear();
    {
        double plus( point.hasErrorY() ? point.errorY().plus_:0 );
        double minus( point.hasErrorY() ? point.errorY().minus_:0 );
        if( point.errorY().relative_ )
        {
            plus *= point.local().y();
            minus *= point.local().y();
        }

        yStatPlus_ = _string( plus, errorFormat );
        yStatMinus_ = _string( minus, errorFormat );
    }

    // systematic errors (horizontal)
    xSystPlus_.clear();
    xSystMinus_.clear();
    {
        double plus( point.hasErrorSystX() ? point.errorSystX().plus_ : 0 );
        double minus( point.hasErrorSystX() ? point.errorSystX().minus_ : 0 );
        if( point.errorSystX().relative_ )
        {
            plus *= point.local().x();
            minus *= point.local().x();
        }

        xSystPlus_ = _string( plus, errorFormat );
        xSystMinus_ = _string( minus, errorFormat );

    }

    // systematic errors (vertical)
    ySystPlus_.clear();
    ySystMinus_.clear();
    {
        double plus( point.hasErrorSystY() ? point.errorSystY().plus_:0 );
        double minus( point.hasErrorSystY() ? point.errorSystY().minus_:0 );
        if( point.errorSystY().relative_ )
        {
            plus *= point.local().y();
            minus *= point.local().y();
        }

        ySystPlus_ = _string( plus, errorFormat );
        ySystMinus_ = _string( minus, errorFormat );
    }

    return *this;

}

//______________________________________________________
QString PrintListDialog::RefPointString::_string( const double& value, const QString& format ) const
{ return format.isEmpty() ? QString::number( value ) : QString().sprintf( qPrintable( format ), value ); }

//______________________________________________________
PrintListDialog::RefPointStringList PrintListDialog::refPointStringList( const RefPoint::List& points, const QString& localFormat, const QString& errorFormat )
{

    RefPointStringList out;
    for( const auto& refPoint:points.get() )
    { out.append( RefPointString().set( refPoint, localFormat, errorFormat ) ); }

    return out;

}
