#ifndef RefPoint_h
#define RefPoint_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "QOrderedSet.h"

#include <QDomDocument>
#include <QDomElement>
#include <QList>
#include <QPair>
#include <QPointF>
#include <QTextStream>

#include <cmath>

//* QPointF comparison
inline bool operator < (const QPointF& first, const QPointF& second )
{
    if( first.x() != second.x() ) return first.x() < second.x();
    else return first.y() < second.y();
}

//* refpoint
class RefPoint final: private Base::Counter<RefPoint>
{

    public:

    //* error mode
    enum ErrorMode
    {
        NoError,
        ErrorSymetric,
        ErrorAsymetric
    };

    //* hover mode
    enum HoverMode
    {

        NoHover,

        // points
        HoverPoint,
        HoverRefPoint,

        // statistical errors
        HoverErrorXPlus,
        HoverErrorXMinus,
        HoverErrorYPlus,
        HoverErrorYMinus,

        // systematic errors
        HoverErrorSystXPlus,
        HoverErrorSystXMinus,
        HoverErrorSystYPlus,
        HoverErrorSystYMinus

    };

    //* drag mode
    enum DragMode
    {

        NoDrag,

        // points
        DragPoint,
        DragRefPoint,

        // statistical errors
        DragErrorXPlus,
        DragErrorXMinus,
        DragErrorYPlus,
        DragErrorYMinus,

        // systematic errors
        DragErrorSystXPlus,
        DragErrorSystXMinus,
        DragErrorSystYPlus,
        DragErrorSystYMinus

    };

    //* drawing mode
    enum DrawFlag
    {
        Markers = 1<<0,
        Lines = 1<<1
    };

    Q_DECLARE_FLAGS( DrawFlags, DrawFlag );


    class Helper
    {

        public:

        //* creator from DOM
        static QPointF pointF( const QDomElement& );

        //* get DOM element
        static QDomElement domElement( const QPointF&, const QString&, QDomDocument& );

        //* length
        static qreal length( const QPointF& point )
        { return std::sqrt( point.x()*point.x() + point.y()*point.y() ); }

        //* distance
        static qreal distance( const QPointF& first, const QPointF& second )
        { return length( second - first ); }

        //* normalize point
        static QPointF normalize( const QPointF& point )
        { return point*( 1.0/length( point ) ); }

        //* rotate point
        /**
        returns rotated point such that x is along argument, and y perpendicular to it.
        assumes that direction.length() is unity
        */
        static const QPointF rotate( const QPointF& source, const QPointF& direction )
        { return QPointF( direction.x()*source.x() + direction.y()*source.y(), -direction.y()*source.x() + direction.x()*source.y() ); }

        //* conversion to string
        static QString toString( const QPointF& source, const QString& = QString() );

    };

    //* errors on point
    class Error final: private Base::Counter<Error>
    {

        public:

        //* constructor
        explicit Error():
            Counter( "RefPoint::Error" )
        {}

        //* creator from DOM
        explicit Error( const QDomElement& );

        //* get DOM element
        QDomElement domElement( const QString&, QDomDocument& ) const;

        //* clear
        void clear()
        { mode_ = NoError; }

        //* validity
        bool isValid() const
        {
            return
                mode_ != NoError &&
                (plus_ != 0 || minus_ != 0) &&
                (globalPlus_ != globalMinus_ );
        }

        ErrorMode mode_ = NoError;
        double plus_ = 0;
        double minus_ = 0;
        bool relative_ = false;

        QPointF globalPlus_;
        QPointF globalMinus_;

    };

    //* container base class
    template<class T>
    class Container
    {
        public:

        //* constructor
        explicit Container()
        {}

        //* constructor
        explicit Container( const T& other ):
            collection_( other )
        {}

        //* destructor
        virtual ~Container() = default;

        //*@name accessors
        //@{

        //* accessor
        const T& get() const { return collection_; }

        //* empty
        bool isEmpty() const { return collection_.isEmpty(); }

        //* true if refPoint is in collection
        bool contains( const RefPoint& point ) const { return collection_.contains( point ); }

        //* size
        int size() const { return collection_.size(); }

        //* draw flags
        DrawFlags drawFlags() const
        { return drawFlags_; }

        //* draw markers
        bool drawMarkers() const
        { return drawFlags_&Markers; }

        //* draw lines
        bool drawLines() const
        { return drawFlags_&Lines; }

        //@}

        //*@name modifiers
        //@{

        //* mutable accessor
        T& get() { return collection_; }


        //* clear
        void clear() { collection_.clear(); }

        //* draw flags
        void setDrawFlags( DrawFlags flags )
        { drawFlags_ = flags; }

        //* draw markers
        bool setDrawMarkers( bool value )
        {
            if( (drawFlags_ & Markers ) == value ) return false;
            else if( value ) drawFlags_ |= Markers;
            else drawFlags_ &= ~Markers;
            return true;
        }

        //* draw markers
        bool setDrawLines( bool value )
        {
            if( (drawFlags_ & Lines ) == value ) return false;
            else if( value ) drawFlags_ |= Lines;
            else drawFlags_ &= ~Lines;
            return true;
        }

        //@}

        private:

        //* draw flags
        DrawFlags drawFlags_ = Markers;

        //* collection
        T collection_;

        //* equal to operator
        friend bool operator == (const Container& first, const Container& second )
        { return first.collection_ == second.collection_ && first.drawFlags_ == second.drawFlags_; }

    };

    //* list of reference points
    class List: public Container<QList<RefPoint>>, private Base::Counter<List>
    {
        public:

        //* default constructor
        explicit List():
            Counter( "RefPoint::List" )
        {}

        //* copy constructor
        explicit List( const QList<RefPoint>& other ):
            Container( other ),
            Counter( "RefPoint::List" )
            {}

        //* creator from DOM
        explicit List( const QDomElement& );

        //*@name accessors
        //@{

        //* get DOM element
        QDomElement domElement( const QString&, QDomDocument& ) const;

        //@}

    };

    //* set of reference points
    class Set: public Container<QOrderedSet<RefPoint>>, private Base::Counter<Set>
    {

        public:

        //* constructor
        explicit Set():
            Counter( "RefPoint::Set" )
            {}

        //* constructor
        explicit Set( const QOrderedSet<RefPoint>& other ):
            Container( other ),
            Counter( "RefPoint::Set" )
        {}

        //*@name modifiers
        //@{

        //* copy from list
        static Set fromList( const List& other )
        {

            Set out( QOrderedSet<RefPoint>::fromList( other.get() ) );
            out.setDrawFlags( other.drawFlags() );
            return out;
        }

        //@}

    };

    //* list iterator
    using ListIterator = QListIterator<RefPoint>;

    //* shortcut to dataset
    using DataSet = List;

    //* shortcut to list of lists
    using Collection = QList<RefPoint::List>;

    //* pair
    using Pair = QPair<RefPoint, RefPoint>;
    using PairList = QList<Pair>;

    //* constructor
    RefPoint():
        Counter( "RefPoint" ),
        valid_( true )
    {}

    //* constructor
    RefPoint( QPointF global, QPointF local ):
        Counter( "RefPoint" ),
        valid_( true ),
        global_( global ),
        local_( local )
    {}

    //* creator from DOM
    RefPoint( const QDomElement& );

    //* get DOM element
    QDomElement domElement( const QString&, QDomDocument& ) const;

    //* equal to operator
    bool operator == (const RefPoint& other ) const
    { return other.global_ == global_; }

    //* less than operatir
    bool operator < (const RefPoint& other ) const
    { return QPointF( global_ ) < other.global_; }

    //*@name accessors
    //@{

    //* global point
    const QPointF& global() const
    { return global_; }

    //* local point
    const QPointF& local() const
    { return local_; }

    //* validity
    bool isValid() const
    { return valid_; }

    //* errors
    bool hasErrorX() const
    { return errorX_.isValid(); }

    //* errors
    bool hasErrorY() const
    { return errorY_.isValid(); }

    //* errors
    bool hasErrors() const
    { return hasErrorX() || hasErrorY(); }

    //* horizontal error
    const Error& errorX() const
    { return errorX_; }

    //* vertical error
    const Error& errorY() const
    { return errorY_; }

    //* errors
    bool hasErrorSystX() const
    { return errorSystX_.isValid(); }

    //* errors
    bool hasErrorSystY() const
    { return errorSystY_.isValid(); }

    //* errors
    bool hasErrorsSyst() const
    { return hasErrorSystX() || hasErrorSystY(); }

    //* horizontal error
    const Error& errorSystX() const
    { return errorSystX_; }

    //* vertical error
    const Error& errorSystY() const
    { return errorSystY_; }

    //* interpolated
    RefPoint interpolated( const RefPoint&, qreal ) const;

    //@}

    //*@name modifiers
    //@{

    //* global point
    bool setGlobal( const QPointF& position )
    {
        if( global_ == position ) return false;
        global_ = position;
        return true;
    }

    //* local point
    bool setLocal( const QPointF& position )
    {
        if( local_ == position ) return false;
        local_ = position;
        return true;
    }

    //* horizontal error
    Error& errorX()
    { return errorX_; }

    //* vertical error
    Error& errorY()
    { return errorY_; }

    //* horizontal systematic error
    Error& errorSystX()
    { return errorSystX_; }

    //* vertical systematic error
    Error& errorSystY()
    { return errorSystY_; }

    //* validity
    void setValid( bool value )
    { valid_ = value; }

    //* clear
    void clear()
    {
        global_ = QPointF();
        local_ = QPointF();

        // errors
        errorX_.clear();
        errorY_.clear();

        // systematc errors
        errorSystX_.clear();
        errorSystY_.clear();
        valid_ = false;
    }

    //@}

    //* find refpoint matching local position
    class SameLocalFTor
    {
        public:

        //* constructor
        explicit SameLocalFTor( const QPointF& local ):
            local_( local )
            {}

        //* prediction
        bool operator () (const RefPoint& other ) const
        { return other.local() == local_; }

        private:

        //* predicate
        QPointF local_;

    };


    //* find refpoint matching global position
    class SameGlobalFTor
    {
        public:

        //* constructor
        explicit SameGlobalFTor( const QPointF& global ):
            global_( global )
            {}

        //* prediction
        bool operator () (const RefPoint& other ) const
        { return other.global() == global_; }

        private:

        //* predicate
        QPointF global_;

    };

    private:

    //* validity flag
    bool valid_ = true;

    //* global point
    QPointF global_;

    //* local point
    QPointF local_;

    //*@name error
    //@{
    Error errorX_;
    Error errorY_;
    Error errorSystX_;
    Error errorSystY_;
    //@}

};

Q_DECLARE_OPERATORS_FOR_FLAGS( RefPoint::DrawFlags )

#endif
