/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "RefPointListWidget.h"

#include "BaseContextMenu.h"
#include "CustomDialog.h"
#include "CustomToolBar.h"
#include "CustomToolButton.h"
#include "IconNames.h"
#include "IconSize.h"
#include "IconEngine.h"
#include "PrintListDialog.h"
#include "QuestionDialog.h"
#include "RefPointFormatDialog.h"
#include "Singleton.h"
#include "XmlOptions.h"

#include <QApplication>
#include <QHeaderView>
#include <QLayout>

//____________________________________________________________
RefPointListWidget::RefPointListWidget( QWidget* parent ):
    QWidget( parent ),
    dataSetIndex_( 0 )
{

    QVBoxLayout* vLayout( new QVBoxLayout );
    vLayout->setMargin(0);
    vLayout->setSpacing(2);
    setLayout( vLayout );

    vLayout->addWidget( dataSetWidget_ = new DataSetWidget( this ) );
    connect( dataSetWidget_, SIGNAL(dataSetAdded()), SLOT(_addDataSet()) );
    connect( dataSetWidget_, SIGNAL(dataSetRemoved()), SLOT(_removeDataSet()) );
    connect( dataSetWidget_, SIGNAL(dataSetSelected(int)), SLOT(_selectDataSet(int)) );

    QHBoxLayout* hLayout( new QHBoxLayout );
    hLayout->setSpacing(0);
    hLayout->setMargin(0);
    vLayout->addLayout( hLayout );

    hLayout->addWidget( list_ = new TreeView( this ), 1 );
    list_->setContextMenuPolicy( Qt::CustomContextMenu );
    list_->setOptionName( "REFPOINT_LIST" );
    list_->setModel( &model_ );
    list_->setSelectionMode( QAbstractItemView::ContiguousSelection );
    list_->setSortingEnabled( false );

    connect( list_->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), SLOT(updateActions()) );
    connect( list_->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), SIGNAL(selectionChanged()) );
    connect( list_, SIGNAL(customContextMenuRequested(QPoint)), SLOT(_updateContextMenu(QPoint)) );

    connect( &model_, SIGNAL(rowsInserted(QModelIndex,int,int)), SLOT(updateActions()) );
    connect( &model_, SIGNAL(rowsRemoved(QModelIndex,int,int)), SLOT(updateActions()) );

    // actions
    _installActions();

    // toolbar
    CustomToolBar* toolbar = new CustomToolBar( tr( "Reference Points" ), this, "REFPOINT_TOOLBAR" );
    toolbar->setMovable( false );
    toolbar->setTransparent( true );
    toolbar->setOrientation( Qt::Vertical );
    toolbar->setIconSize( IconSize::get( IconSize::Small ) );

    toolbar->addAction( removeAction_ );
    toolbar->addAction( moveUpAction_ );
    toolbar->addAction( moveDownAction_ );
    toolbar->addAction( sortAction_ );
    toolbar->addAction( printAction_ );
    // toolbar->addAction( refPointFormatAction_ );

    // create
    CustomToolButton* toolButton = new CustomToolButton( toolbar );
    toolButton->setIcon( IconEngine::get( IconNames::Configure ) );
    toolButton->setText( tr( "Tools" ) );

    toolButton->setPopupMode( QToolButton::InstantPopup );
    QMenu* menu = new QMenu;
    toolButton->setMenu( menu );
    menu->addAction( refPointFormatAction_ );
    menu->addAction( regularizeAction_ );
    menu->addSeparator();
    menu->addAction( drawMarkersAction_ );
    menu->addAction( drawLinesAction_ );

    toolbar->addWidget( toolButton );

    hLayout->addWidget( toolbar, 0 );

    updateActions();

    connect( Base::Singleton::get().application(), SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    connect( this, SIGNAL(refPointFormatChanged()), SLOT(_updateConfiguration()) );

    _updateConfiguration();

}

//____________________________________________________________
RefPoint::List RefPointListWidget::get() const
{
    RefPoint::List points( model_.get() );
    points.setDrawMarkers( drawMarkersAction_->isChecked() );
    points.setDrawLines( drawLinesAction_->isChecked() );
    return points;
}


//____________________________________________________________
RefPoint::Collection RefPointListWidget::collection() const
{

    RefPoint::Collection out( dataSets_ );
    if( dataSetIndex_ < (int) out.size() )
    { out[dataSetIndex_] = get(); }

    return out;
}

//____________________________________________________________
bool RefPointListWidget::hasSelection() const
{ return !list_->selectionModel()->selectedRows().isEmpty(); }

//____________________________________________________________
RefPoint::List RefPointListWidget::selection() const
{
    auto selectedIndexes( list_->selectionModel()->selectedRows() );
    return RefPoint::List( model_.get( selectedIndexes ) );
}

//____________________________________________________________
void RefPointListWidget::setCollection( const RefPoint::Collection& collection )
{

    Debug::Throw( "RefPointListWidget::setCollection.\n" );

    dataSets_ = collection;
    if( dataSetIndex_ < (int) dataSets_.size() )
    {

        const RefPoint::List& points( dataSets_[dataSetIndex_] );
        model_.set( points.get() );
        list_->resizeColumns();

        // update draw options
        drawMarkersAction_->setChecked( points.drawMarkers() );
        drawLinesAction_->setChecked( points.drawLines() );

    } else model_.clear();

    // clear selection
    list_->clearSelection();

    // update dataset selection widget
    dataSetWidget_->setCollection( collection );

}

//____________________________________________________________
void RefPointListWidget::addDataSet( const RefPoint::List& points )
{

    Debug::Throw( "RefPointListWidget::addDataSet.\n" );

    // store in collection
    dataSets_.append( points );

    // pass to model if index matches
    if( (int) dataSets_.size() == dataSetIndex_ + 1 )
    {
        model_.set( points.get() );
        list_->resizeColumns();

        // adjust markers action
        drawMarkersAction_->setChecked( points.drawMarkers() );
        drawLinesAction_->setChecked( points.drawLines() );

    }

}

//____________________________________________________________
void RefPointListWidget::setDataSetIndex( int value )
{

    Debug::Throw() << "RefPointListWidget::setDataSetIndex - " << value << endl;

    // check if changed
    if( value == dataSetIndex_ ) return;

    // first store current dataset if valid
    if( dataSetIndex_ < (int) dataSets_.size() )
    { dataSets_[dataSetIndex_] = get(); }

    // update index
    dataSetIndex_ = value;

    // update model
    model_.clear();
    if( dataSetIndex_ < (int) dataSets_.size() )
    {
        const auto& points( dataSets_[dataSetIndex_] );
        model_.set( points.get() );
        list_->resizeColumns();
        drawMarkersAction_->setChecked( points.drawMarkers() );
        drawLinesAction_->setChecked( points.drawLines() );
    }

    // clear selection
    list_->clearSelection();

    // update dataset widget
    dataSetWidget_->select( dataSetIndex_ );

}

//____________________________________________________________
void RefPointListWidget::clear()
{
    Debug::Throw( "RefPointListWidget::clear.\n" );
    dataSets_.clear();
    model_.clear();
    clipboard_ = RefPoint();
    emit clipboardAvailable( false );
}

//____________________________________________________________
void RefPointListWidget::add( const RefPoint& point )
{
    Debug::Throw( "RefPointListWidget::add.\n" );
    model_.add( point );
    list_->resizeColumns();
}

//____________________________________________________________
void RefPointListWidget::insert( int refPointIndex, const RefPoint& point )
{
    Debug::Throw( "RefPointListWidget::add.\n" );
    const QModelIndex index( model_.index( refPointIndex, 0 ) );
    if( index.isValid() ) model_.insert( index, point );
    else model_.add( point );
    list_->resizeColumns();
}

//____________________________________________________________
void RefPointListWidget::remove( const RefPoint& point )
{
    Debug::Throw( "RefPointListWidget::remove.\n" );
    model_.remove( point );
    list_->resizeColumns();
}

//____________________________________________________________
void RefPointListWidget::copy( const RefPoint& point )
{
    Debug::Throw( "RefPointListWidget::copy.\n" );
    clipboard_ = point;
    emit clipboardAvailable( true );
}

//____________________________________________________________
void RefPointListWidget::clearSelection()
{
    Debug::Throw( "RefPointListWidget::clearSelection.\n" );
    list_->clearSelection();
}

//____________________________________________________________
bool RefPointListWidget::select( const RefPoint& point )
{
    Debug::Throw( "RefPointListWidget::select.\n" );

    const QModelIndex index( model_.index( point ) );
    if( index.isValid() )
    {

        list_->clearSelection();
        list_->selectionModel()->select( index, QItemSelectionModel::Select|QItemSelectionModel::Current|QItemSelectionModel::Rows );
        return true;

    } else {

        // try find inactive point
        for( int index = 0; index < (int) dataSets_.size(); index++ )
        {

            // skip current data set
            if( index == dataSetIndex_ ) continue;
            if( dataSets_[index].contains( point ) )
            {
                setDataSetIndex( index );
                select( point );
                return true;
            }
        }

    }

    return false;

}

//____________________________________________________________
void RefPointListWidget::movePointsUp( const RefPoint::List& selectedRefPoints )
{

    Debug::Throw( "RefPointListWidget::movePointsUp.\n" );

    QList<RefPoint> newRefPoints;
    for( const auto& current:model_.get() )
    {

        // check if new list is not empty, current index is selected and last index is not.
        // if yes, move.
        if(
            !( newRefPoints.empty() ||
            selectedRefPoints.contains( newRefPoints.back() ) ||
            !selectedRefPoints.contains( current ) ) )
        {
            RefPoint last( newRefPoints.back() );
            newRefPoints.removeLast();
            newRefPoints.append( current );
            newRefPoints.append( last );
        } else newRefPoints << current;

    }

    model_.set( newRefPoints );

    // restore selection
    list_->selectionModel()->select( model_.index( selectedRefPoints.get().front() ),  QItemSelectionModel::Clear|QItemSelectionModel::Select|QItemSelectionModel::Rows );
    for( const auto& refPoint:selectedRefPoints.get() )
    { list_->selectionModel()->select( model_.index( refPoint ), QItemSelectionModel::Select|QItemSelectionModel::Rows ); }

    return;

}

//____________________________________________________________
void RefPointListWidget::movePointsDown( const RefPoint::List& selectedRefPoints )
{

    Debug::Throw( "RefPointListWidget::movePointsDown.\n" );

    QList<RefPoint> newRefPoints;
    RefPoint::ListIterator iter( model_.get() );
    iter.toBack();
    while( iter.hasPrevious() )
    {
        const RefPoint& current( iter.previous() );
        if(
            !( newRefPoints.isEmpty() ||
            selectedRefPoints.contains( newRefPoints.front() ) ||
            !selectedRefPoints.contains( current ) ) )
        {

            RefPoint first( newRefPoints.front() );
            newRefPoints.removeFirst();
            newRefPoints.prepend( current );
            newRefPoints.prepend( first );

        } else newRefPoints.prepend( current );
    }

    model_.set( newRefPoints );

    // restore selection
    list_->selectionModel()->select( model_.index( selectedRefPoints.get().front() ),  QItemSelectionModel::Clear|QItemSelectionModel::Select|QItemSelectionModel::Rows );
    for( const auto& refPoint:selectedRefPoints.get() )
    { list_->selectionModel()->select( model_.index( refPoint ), QItemSelectionModel::Select|QItemSelectionModel::Rows ); }

}

//____________________________________________________________
bool RefPointListWidget::replace( const RefPoint& source, const RefPoint& destination )
{

    Debug::Throw( "RefPointListWidget::replace.\n" );
    QModelIndex index( model_.index( source ) );
    if( index.isValid() )
    {
        model_.replace( index, destination );
        list_->resizeColumns();
        return true;
    } else return false;
}

//____________________________________________________________
bool RefPointListWidget::replace( int dataSetIndex, const RefPoint& source, const RefPoint& destination )
{

    if( dataSetIndex == dataSetIndex_ ) return replace( source, destination );
    else if( dataSetIndex < dataSets_.size() )
    {

        // replace in collection directly
        auto& referencePoints( dataSets_[dataSetIndex].get() );
        const int index( referencePoints.indexOf( source ) );
        if( index < 0 ) return false;
        else {

            referencePoints[index] = destination;
            return true;

        }

    } else return false;

}

//____________________________________________________________
void RefPointListWidget::updateActions()
{

    Debug::Throw( "RefPointListWidget::updateActions.\n" );

    QModelIndexList selection( list_->selectionModel()->selectedRows() );
    const bool hasSelection( !selection.empty() );

    const bool hasBefore( hasSelection && selection.front().row() > 0 );
    const bool hasAfter( hasSelection && selection.back().row() < model_.rowCount()-1 );
    removeAction_->setEnabled( hasSelection );
    moveUpAction_->setEnabled( hasBefore );
    moveDownAction_->setEnabled( hasAfter );

    // print action
    bool modelEmpty( model_.rowCount() == 0 );
    printAction_->setEnabled( !modelEmpty );
    printAction_->setText( dataSets_.size() > 1 ? tr( "Print Active Marker Set" ):tr( "Print Markers" ) );

    // print all action
    bool collectionEmpty( true );
    collectionEmpty &= modelEmpty;
    if( collectionEmpty )
    {
        for( int index = 0; index < (int) dataSets_.size(); index++ )
        {
            if( index == dataSetIndex_ ) continue;
            if( !dataSets_[index].isEmpty() ) { collectionEmpty = false; break; }
        }
    }

    printAllAction_->setEnabled( !collectionEmpty );
    printAllAction_->setVisible( dataSets_.size() > 1 );

}

//____________________________________________________________
void RefPointListWidget::_updateContextMenu( const QPoint& position )
{

    Debug::Throw( "RefPointListWidget::updateErrorsActions.\n" );

    QModelIndex current( list_->selectionModel()->currentIndex() );
    const bool hasCurrent( current.isValid() );
    editErrorsAction_->setEnabled( hasCurrent );

    RefPoint currentPoint( current.isValid() ? model_.get( current ):RefPoint() );

    defaultErrorsAction_->setEnabled(
        hasCurrent &&
        ((!currentPoint.hasErrorX() && XmlOptions::get().get<int>( "DEFAULT_ERROR_X_MODE" ) != RefPoint::NoError ) ||
        (!currentPoint.hasErrorY() && XmlOptions::get().get<int>( "DEFAULT_ERROR_Y_MODE" ) != RefPoint::NoError ) )
        );

    defaultErrorsSystAction_->setEnabled(
        hasCurrent &&
        ((!currentPoint.hasErrorSystX() && XmlOptions::get().get<int>( "DEFAULT_ERROR_SYST_X_MODE" ) != RefPoint::NoError ) ||
        (!currentPoint.hasErrorSystY() && XmlOptions::get().get<int>( "DEFAULT_ERROR_SYST_Y_MODE" ) != RefPoint::NoError ) )
        );

    BaseContextMenu menu( list_ );
    menu.setHideDisabledActions( true );
    menu.addAction( removeAction_ );
    menu.addAction( moveUpAction_ );
    menu.addAction( moveDownAction_ );
    menu.addAction( sortAction_ );
    menu.addSeparator();
    menu.addAction( editErrorsAction_ );
    menu.addAction( defaultErrorsAction_ );
    menu.addAction( defaultErrorsSystAction_ );

    // execute
    menu.exec( list_->viewport()->mapToGlobal( position ) );

}

//_______________________________________________________
void RefPointListWidget::_toggleDrawMarkers( bool value )
{

    // check modified and update
    if( dataSetIndex_ < dataSets_.size() && dataSets_[dataSetIndex_].drawMarkers() != value )
    {
        dataSets_[dataSetIndex_].setDrawMarkers( value );
        emit modified();
    }

    // make sure at least one draw options is enabled and adjust action's enability
    if( !value ) { drawLinesAction_->setChecked( true ); }
    drawLinesAction_->setEnabled( value );
    drawMarkersAction_->setEnabled( drawLinesAction_->isChecked() );
}

//_______________________________________________
void RefPointListWidget::_toggleDrawLines( bool value )
{
    if( dataSetIndex_ < dataSets_.size() && dataSets_[dataSetIndex_].drawLines() != value )
    {
        dataSets_[dataSetIndex_].setDrawLines( value );
        emit modified();
    }

    // make sure at least one draw options is enabled and adjust action's enability
    if( !value ) { drawMarkersAction_->setChecked( true ); }
    drawMarkersAction_->setEnabled( value );
    drawLinesAction_->setEnabled( drawMarkersAction_->isChecked() );
}

//____________________________________________________________
void RefPointListWidget::_moveUp()
{
    Debug::Throw( "RefPointListWidget::_moveUp.\n" );

    // retrieve selected indexes in list and store in model
    auto selectedIndexes( list_->selectionModel()->selectedRows() );
    if( !selectedIndexes.empty() )
    { emit selectionMovedUp( RefPoint::List( model_.get( selectedIndexes ) ) ); }

}

//____________________________________________________________
void RefPointListWidget::_moveDown()
{
    Debug::Throw( "RefPointListWidget::_moveDown.\n" );

    // retrieve selected indexes in list and store in model
    auto selectedIndexes( list_->selectionModel()->selectedIndexes() );
    if( !selectedIndexes.empty() )
    { emit selectionMovedDown( RefPoint::List( model_.get( selectedIndexes ) ) ); }

}

//____________________________________________________________
void RefPointListWidget::_sort()
{
    Debug::Throw( "RefPointListWidget::_sort.\n" );

    // retrieve selected indexes in list and store in model
    QModelIndexList selectedIndexes( list_->selectionModel()->selectedIndexes() );
    auto selectedRefPoints( model_.get( selectedIndexes ) );

    // store current ref points, sort and reset.
    auto currentRefPoints( model_.get() );
    std::sort( currentRefPoints.begin(), currentRefPoints.end() );
    model_.set( currentRefPoints );

    // restore selection
    if( !selectedRefPoints.isEmpty() )
    {
        list_->selectionModel()->select( model_.index( selectedRefPoints.front() ),  QItemSelectionModel::Clear|QItemSelectionModel::Select|QItemSelectionModel::Rows );
        for( const auto& refPoint:selectedRefPoints )
        { list_->selectionModel()->select( model_.index( refPoint ), QItemSelectionModel::Select|QItemSelectionModel::Rows ); }
    }

}

//____________________________________________________________
void RefPointListWidget::_remove()
{ emit selectionRemoved( RefPoint::List( model_.get( list_->selectionModel()->selectedRows() ) ) ); }

//____________________________________________________________
void RefPointListWidget::_print()
{

    Debug::Throw( "RefPointListWidget::_print.\n" );
    PrintListDialog dialog( this );
    dialog.setCollection( collection(), dataSetIndex_ );

    // connections
    connect( &dialog.formatToolButton(), SIGNAL(clicked()), SLOT(_configureRefPointFormat()) );
    connect( this, SIGNAL(refPointFormatChanged()), &dialog, SLOT(updateText()) );

    dialog.exec();

}

//____________________________________________________________
void RefPointListWidget::_printAll()
{

    Debug::Throw( "RefPointListWidget::_printAll.\n" );
    PrintListDialog dialog( this );
    dialog.setCollection( collection() );
    dialog.exec();

}

//____________________________________________________________
void RefPointListWidget::_editErrors()
{

    Debug::Throw( "RefPointListWidget::_editErrors.\n" );
    const QModelIndex index( list_->selectionModel()->currentIndex() );
    if( !index.isValid() ) return;
    emit errorsEdited( model_.get( index ) );

}

//____________________________________________________________
void RefPointListWidget::_addDefaultErrors()
{

    Debug::Throw( "RefPointListWidget::_addDefaultErrors.\n" );
    const QModelIndex index( list_->selectionModel()->currentIndex() );
    if( !index.isValid() ) return;
    emit defaultErrorsAdded( model_.get( index ) );

}

//____________________________________________________________
void RefPointListWidget::_addDefaultErrorsSyst()
{

    Debug::Throw( "RefPointListWidget::_addDefaultErrorsSyst.\n" );
    const QModelIndex index( list_->selectionModel()->currentIndex() );
    if( !index.isValid() ) return;
    emit defaultErrorsSystAdded( model_.get( index ) );

}

//____________________________________________________________
void RefPointListWidget::_addDataSet()
{

    Debug::Throw( "RefPointListWidget::_addDataSet.\n" );
    dataSets_ << RefPoint::List();
    dataSetWidget_->setCollection( dataSets_ );
    dataSetWidget_->select( dataSets_.size()-1 );
    updateActions();
    emit modified();

}

//____________________________________________________________
void RefPointListWidget::_removeDataSet()
{

    Debug::Throw( "RefPointListWidget::_removeDataSet.\n" );
    if( dataSetIndex_ >= (int) dataSets_.size() ) return;
    if( dataSets_.size() < 2 ) return;

    if( !QuestionDialog( this,
        tr( "Remove current data set ?\n"
        "Warning: all associated markers will be definitly lost" ) ).exec() ) return;

    RefPoint::Collection local( dataSets_ );
    dataSets_.clear();
    for( int index = 0; index < (int) local.size(); index++ )
    { if( index != dataSetIndex_ ) dataSets_ << local[index]; }

    const int oldIndex( dataSetIndex_ );

    dataSetWidget_->setCollection( dataSets_ );

    // update selection
    const int index( qMin( oldIndex, (int)dataSets_.size()-1 ) );
    dataSetWidget_->select( index );
    _selectDataSet( index );

    updateActions();
    emit modified();

}

//____________________________________________________________
void RefPointListWidget::_selectDataSet( int index )
{

    Debug::Throw() << "RefPointListWidget::_selectDataSet - index: " << index << endl;
    if( index < 0 || index >= (int) dataSets_.size() ) return;
    setDataSetIndex( index );
    emit modified();

}

//____________________________________________________________
void RefPointListWidget::_configureRefPointFormat()
{

    Debug::Throw( "RefPointListWidget::_configureRefPointFormat.\n" );
    RefPointFormatDialog dialog( this );

    // connections
    connect( &dialog.applyButton(), SIGNAL(clicked()), this, SIGNAL(refPointFormatChanged()) );
    connect( &dialog.okButton(), SIGNAL(clicked()), this, SIGNAL(refPointFormatChanged()) );

    dialog.centerOnWidget( qApp->activeWindow() ).exec();

}

//____________________________________________________________
void RefPointListWidget::_installActions()
{

    addAction( moveUpAction_ = new QAction( IconEngine::get( IconNames::Up ), tr( "Move Selection Up" ), this ) );
    connect( moveUpAction_, SIGNAL(triggered()), SLOT(_moveUp()) );

    addAction( moveDownAction_ = new QAction( IconEngine::get( IconNames::Down ), tr( "Move Selection Down" ), this ) );
    connect( moveDownAction_, SIGNAL(triggered()), SLOT(_moveDown()) );

    addAction( sortAction_ = new QAction( IconEngine::get( IconNames::Sort ), tr( "Sort Markers" ), this ) );
    connect( sortAction_, SIGNAL(triggered()), SLOT(_sort()) );

    addAction( removeAction_ = new QAction( IconEngine::get( IconNames::Remove ), tr( "Remove Selection" ), this ) );
    connect( removeAction_, SIGNAL(triggered()), SLOT(_remove()) );

    addAction( printAction_ = new QAction( IconEngine::get( IconNames::Print ),tr(  "Print Markers" ), this ) );
    connect( printAction_, SIGNAL(triggered()), SLOT(_print()) );

    addAction( printAllAction_ = new QAction( IconEngine::get( IconNames::Print ), tr( "Print All Markers" ), this ) );
    connect( printAllAction_, SIGNAL(triggered()), SLOT(_printAll()) );

    addAction( editErrorsAction_ = new QAction( IconEngine::get( IconNames::Edit ), tr( "Edit Marker Uncertainties" ), this ) );
    connect( editErrorsAction_, SIGNAL(triggered()), SLOT(_editErrors()) );

    addAction( defaultErrorsAction_ = new QAction( IconEngine::get( IconNames::Edit ), tr( "Add Default Statistical Uncertainties to Marker" ), this ) );
    connect( defaultErrorsAction_, SIGNAL(triggered()), SLOT(_addDefaultErrors()) );

    addAction( defaultErrorsSystAction_ = new QAction( IconEngine::get( IconNames::Edit ), tr( "Add Default Systematic Uncertainties to Marker" ), this ) );
    connect( defaultErrorsSystAction_, SIGNAL(triggered()), SLOT(_addDefaultErrorsSyst()) );

    addAction( refPointFormatAction_ = new QAction( IconEngine::get( IconNames::Configure ), tr( "Configure Marker Format for Printing..." ), this ) );
    connect( refPointFormatAction_, SIGNAL(triggered()), SLOT(_configureRefPointFormat()) );

    addAction( regularizeAction_ = new QAction( tr( "Regularize" ), this ) );
    regularizeAction_->setToolTip( tr( "Replace existing markers with a set of markers that are evenly spaced along x axis" ) );

    addAction( drawMarkersAction_ = new QAction( tr( "Draw Marker Symbols" ), this ) );
    drawMarkersAction_->setCheckable( true );
    connect( drawMarkersAction_, SIGNAL(toggled(bool)), SLOT(_toggleDrawMarkers(bool)) );

    addAction( drawLinesAction_ = new QAction( tr( "Draw Lines Between Markers" ), this ) );
    drawLinesAction_->setCheckable( true );
    connect( drawLinesAction_, SIGNAL(toggled(bool)), SLOT(_toggleDrawLines(bool)) );

}

//______________________________________
void RefPointListWidget::_updateConfiguration()
{
    Debug::Throw( "RefPointListWidget::_updateConfiguration.\n" );
    model_.setGlobalFormat( XmlOptions::get().get<QString>( "GLOBAL_FORMAT" ) );
    model_.setLocalFormat( XmlOptions::get().get<QString>( "LOCAL_FORMAT" ) );
    model_.setErrorFormat( XmlOptions::get().get<QString>( "ERROR_FORMAT" ) );
    list_->resizeColumns();
}
