#ifndef RefPointListWidget_h
#define RefPointListWidget_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "DataSetWidget.h"
#include "RefPointModel.h"
#include "TreeView.h"

#include <QAction>
#include <QWidget>

class RefPointListWidget: public QWidget
{

    Q_OBJECT

    public:

    //* constructor
    explicit RefPointListWidget( QWidget* = nullptr );

    //*@name accessors
    //@{

    //* points (for current dataSet)
    RefPoint::List get() const;

    //* points (from collection)
    RefPoint::Collection collection() const;

    //* has selection
    bool hasSelection() const;

    //* selection points
    RefPoint::List selection() const;

    //* dataset index
    int dataSetIndex() const
    { return dataSetIndex_; }

    //* return marker index in current dataset
    int index( const RefPoint& point ) const
    {
        const QModelIndex index( model_.index( point ) );
        return index.isValid() ? index.row():-1;
    }

    //* true if current dataset contains point
    bool contains( const RefPoint& point ) const
    { return model_.index( point ).isValid(); }

    //@}

    //*@name modifiers
    //@{

    //* set collection
    void setCollection( const RefPoint::Collection& );

    //* add data set
    void addDataSet( const RefPoint::List& );

    //* dataSet index
    void setDataSetIndex( int );

    //* clear
    void clear();

    //* add
    void add( const RefPoint& );

    //* insert
    void insert( int, const RefPoint& );

    //* remove
    void remove( const RefPoint& );

    //* copy
    void copy( const RefPoint& );

    //* paste
    const RefPoint& clipboard() const
    { return clipboard_; }

    //* clear selection
    void clearSelection();

    //* select point
    bool select( const RefPoint& );

    //* move selection up
    void movePointsUp( const RefPoint::List& );

    //* move selection down
    void movePointsDown( const RefPoint::List& );

    //* replace two markers
    bool replace( const RefPoint&, const RefPoint& );

    //* replace two markers from given dataset
    bool replace( int, const RefPoint&, const RefPoint& );

    //* validity
    void setRefPointsValid( bool value )
    { model_.setRefPointsValid( value ); }

    //@}

    //*@name actions
    //@{

    QAction& moveUpAction() const
    { return *moveUpAction_; }

    QAction& moveDownAction() const
    { return *moveDownAction_; }

    QAction& sortAction() const
    { return *sortAction_; }

    QAction& removeAction() const
    { return *removeAction_; }

    QAction& printAction() const
    { return *printAction_; }

    QAction& printAllAction() const
    { return *printAllAction_; }

    QAction& editErrorsAction() const
    { return *editErrorsAction_; }

    QAction& defaultErrorsAction() const
    { return *defaultErrorsAction_; }

    QAction& defaultErrorsSystAction() const
    { return *defaultErrorsSystAction_; }

    QAction& refPointFormatAction() const
    { return *refPointFormatAction_; }

    QAction& regularizeAction() const
    { return *regularizeAction_; }

    //@}

    Q_SIGNALS:

    //* emit when modified
    void modified();

    //* emit when selection is changed
    void selectionChanged();

    //* emit when selection is removed
    void selectionRemoved( const RefPoint::List& );

    //* emit when selection is moved up
    void selectionMovedUp( const RefPoint::List& );

    //* emit when selection is moved down
    void selectionMovedDown( const RefPoint::List& );

    //* emit when error edition is requested
    void errorsEdited( RefPoint );

    //* emit when default errors has to be added
    void defaultErrorsAdded( RefPoint );

    //* syst errors to be added
    void defaultErrorsSystAdded( RefPoint );

    //* clipboard
    void clipboardAvailable( bool );

    // format configuration
    void refPointFormatChanged();

    public Q_SLOTS:

    //* update actions
    void updateActions();

    protected Q_SLOTS:

    //* move selection up
    void _moveUp();

    //* move selection down
    void _moveDown();

    //* sort
    void _sort();

    //* remove
    void _remove();

    //* print
    void _print();

    //* print
    void _printAll();

    //* edit errors
    void _editErrors();

    //* add default errors
    void _addDefaultErrors();

    //* add default syst errors
    void _addDefaultErrorsSyst();

    //* add data set
    void _addDataSet();

    //* remove data set
    void _removeDataSet();

    //* select data set
    void _selectDataSet( int );

    //* reference point format configuration
    void _configureRefPointFormat();

    //* update context menu
    void _updateContextMenu( const QPoint& );

    //* toggle draw marker symbols
    void _toggleDrawMarkers( bool );

    //* toggle draw lines symbols
    void _toggleDrawLines( bool );

    private Q_SLOTS:

    //* update configuration
    void _updateConfiguration();

    private:

    //* install actions
    void _installActions();

    //* Ref point collection
    RefPoint::Collection dataSets_;

    //* copied point
    RefPoint clipboard_;

    //* index
    int dataSetIndex_ = 0;

    //*@name actions
    //@{

    QAction* moveUpAction_ = nullptr;
    QAction* moveDownAction_ = nullptr;
    QAction* sortAction_ = nullptr;
    QAction* removeAction_ = nullptr;
    QAction* printAction_ = nullptr;
    QAction* printAllAction_ = nullptr;
    QAction* editErrorsAction_ = nullptr;
    QAction* defaultErrorsAction_ = nullptr;
    QAction* defaultErrorsSystAction_ = nullptr;
    QAction* refPointFormatAction_ = nullptr;

    QAction* regularizeAction_ = nullptr;
    QAction* drawMarkersAction_ = nullptr;
    QAction* drawLinesAction_ = nullptr;

    //@}

    //* model
    RefPointModel model_;

    //* dataset widget
    DataSetWidget* dataSetWidget_ = nullptr;

    //* list
    TreeView* list_ = nullptr;

};

#endif
