/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "RefPointModel.h"
#include "Debug.h"

#include <QPalette>
#include <QTextStream>

//__________________________________________________________________
QVariant RefPointModel::data( const QModelIndex& index, int role ) const
{
    // check index
    if( !contains( index ) ) return QVariant();

    // retrieve associated file info
    const RefPoint& point( get(index) );

    // return text associated to file and column
    if( role == Qt::DisplayRole ) {

        switch( index.column() )
        {

            case Index: return index.row();

            case Global: return RefPoint::Helper::toString( point.global(), globalFormat_ );

            case Local: return RefPoint::Helper::toString( point.local(), localFormat_ );

            case ErrorsX:
            if( point.hasErrorX() ) return error( point.errorX(), point.local().x() );
            else return QVariant();

            case ErrorsY:
            if( point.hasErrorY() ) return error( point.errorY(), point.local().y() );
            else return QVariant();

            case ErrorsSystX:
            if( point.hasErrorSystX() ) return error( point.errorSystX(), point.local().x() );
            else return QVariant();

            case ErrorsSystY:
            if( point.hasErrorSystY() ) return error( point.errorSystY(), point.local().y() );
            else return QVariant();

            default:
            return QVariant();

        }

    } else if( role == Qt::ForegroundRole ) {

        if( !refPointsValid_ ) return QPalette().color( QPalette::Disabled, QPalette::Text );
        else return QVariant();

    } else if( role == Qt::TextAlignmentRole ) {

        switch( index.column() )
        {

            case Index:
            return (int)(Qt::AlignRight|Qt::AlignVCenter);

            default: return QVariant();

        }

    }

    return QVariant();

}

//__________________________________________________________________
QVariant RefPointModel::headerData(int section, Qt::Orientation orientation, int role) const
{

    if( role == Qt::DisplayRole )
    {
        if( section >= 0 && section < nColumns )
        { return columnTitles_[section]; }

    } else if( role == Qt::TextAlignmentRole ) {

        switch( section )
        {

            case Index:
            return (int)(Qt::AlignRight|Qt::AlignVCenter);

            default: return QVariant();

        }

    }

    // return empty
    return QVariant();

}

//__________________________________________________________________
QString RefPointModel::error( const RefPoint::Error& error, const double& value ) const
{

    double plus( error.plus_ );
    double minus( error.minus_ );
    if( error.relative_ )
    {
        plus *= value;
        minus *= value;
    }

    if( errorFormat_.isEmpty() )
    {

        return QString( "( +%1, -%2 )" ).arg( plus ).arg( minus );

    } else {

        return QString( "( +%1, -%2 )" )
            .arg( QString().sprintf( qPrintable( errorFormat_ ), plus ) )
            .arg( QString().sprintf( qPrintable( errorFormat_ ), minus ) );

    }

}

//__________________________________________________________________
void RefPointModel::setGlobalFormat( const QString& format )
{
    // store format
    globalFormat_ = format;
    emit layoutChanged();

}

//__________________________________________________________________
void RefPointModel::setLocalFormat( const QString& format )
{
    // store format
    localFormat_ = format;
    emit layoutChanged();

}

//__________________________________________________________________
void RefPointModel::setErrorFormat( const QString& format )
{
    // store format
    errorFormat_ = format;
    emit layoutChanged();

}
