#ifndef RefPointPanel_h
#define RefPointPanel_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "RefPointWidget.h"

#include <QAbstractItemView>
#include <QActionGroup>
#include <QCheckBox>
#include <QFrame>
#include <QLayout>
#include <QPainter>
#include <QPaintEvent>
#include <QWidget>
#include <QVector>

class ZoomWidget;

class RefPointPanel: public QWidget, private Base::Counter<RefPointPanel>
{

    Q_OBJECT

    public:

    //* number of ref points
    enum { RefPointCount = 3 };

    //* constructor
    explicit RefPointPanel( QWidget* = nullptr );

    //*@name accessors
    //@{

    //* button group
    QActionGroup& editActions() const
    { return *editActions_; }

    //* button group
    QActionGroup& clearActions() const
    { return *clearActions_; }

    // has hover
    bool hasHover() const
    {
        for( int index=0; index<RefPointCount; index++ )
        { if( refPointWidgets_[index]->isHovered() ) return true; }

        return false;
    }

    // has selection
    bool hasSelection() const
    {
        for( int index=0; index<RefPointCount; index++ )
        { if( refPointWidgets_[index]->isSelected() ) return true; }

        return false;
    }

    //* selected point
    RefPoint selection() const
    {
        for( int index=0; index<RefPointCount; index++ )
        { if( refPointWidgets_[index]->isSelected() ) return refPointWidgets_[index]->point(); }
        return RefPoint();
    }

    //* selected index
    int selectedIndex() const
    {
        for( int index=0; index<RefPointCount; index++ )
        { if( refPointWidgets_[index]->isSelected() ) return index; }
        return -1;
    }

    //* get point for index
    const RefPoint& point( int index ) const
    { return refPointWidgets_[index]->point(); }

    //* get index for point
    int index( const RefPoint& point ) const
    {

        for( int index = 0; index < RefPointCount; ++index )
        { if( refPointWidgets_[index]->point() == point ) return index; }

        return -1;

    }

    //* get list of reference points
    RefPoint::List points() const
    {
        RefPoint::List out;
        for( int index=0; index<RefPointCount; index++ )
        { if( refPointWidgets_[index]->isValid() ) out.get().append( point(index) ); }

        return out;
    }

    //* true when valid
    bool isValid() const
    { return validate() == Valid; }

    //* get first invalid index
    int firstInvalidIndex() const;

    //* true if refPoint is in list
    bool contains( const RefPoint& point ) const
    { return index( point ) >= 0; }

    //* transformation
    QPointF toLocal( const QPointF& ) const;

    //* transformation
    QPointF toGlobal( const QPointF& ) const;

    //* transformation
    RefPoint toLocal( const RefPoint& ) const;

    //* transformation
    RefPoint toGlobal( const RefPoint& ) const;

    //* true if log x selected
    bool hasLogX() const
    { return logXCheckBox_->isChecked(); }

    //* true if log x selected
    bool hasLogY() const
    { return logYCheckBox_->isChecked(); }

    //* zoom widget
    ZoomWidget& zoomWidget() const
    { return *zoomWidget_; }

    //@}

    //*@name modifiers
    //@{

    //* select
    void select( const RefPoint& );

    //* select index
    bool select( int );

    //* clear selection
    void clearSelection();

    //* set reference point at index
    void setPoint( int index, const RefPoint& refPoint );

    //* validation error code
    enum ErrorCode
    {
        Valid,
        Incomplete,
        LogXInvalid,
        LogYInvalid,
        GlobalInvalid,
        LocalInvalid
    };

    ErrorCode validate() const;

    //* toggle log x
    void setLogX( bool value )
    { logXCheckBox_->setChecked( value ); }

    //* toggle log y
    void setLogY( bool value )
    { logYCheckBox_->setChecked( value ); }

    //* clear
    void clear( int index )
    { refPointWidgets_[index]->clear(); }

    //* clear
    void clear()
    {
        clearSelection();
        for( const auto& widget:refPointWidgets_ )
        { widget->clear(); }
    }

    //* remove
    bool remove( const RefPoint& );

    //* event filter (used to select refpoint by left clicking
    bool eventFilter( QObject*, QEvent* );

    //@}

    Q_SIGNALS:

    //* emitted when changed
    void modified();

    //* emitted when selection has changed
    void selectionChanged();

    protected Q_SLOTS:

    //* update transormation
    void _updateTransformation();

    //* context menu
    void _updateContextMenu( const QPoint& );

    protected:

    //* paint event
    void paintEvent( QPaintEvent* ) override;

    //* mouse press event
    void mousePressEvent( QMouseEvent* event ) override
    {
        clearSelection();
        emit selectionChanged();
        return QWidget::mousePressEvent( event );
    }

    //* determinant (global coordinates)
    double _globalDeterminant() const;

    //* determinant (local coordinates)
    double _localDeterminant() const;

    private:

    //* some styles require an item view passed to painting method to have proper selection rendered in items
    QAbstractItemView* itemView_ = nullptr;

    //* button group
    QActionGroup* editActions_ = nullptr;

    //* button group
    QActionGroup* clearActions_ = nullptr;

    //* zoom Widget
    ZoomWidget* zoomWidget_ = nullptr;

    //*@name checkboxes
    //@{
    QCheckBox* logXCheckBox_ = nullptr;
    QCheckBox* logYCheckBox_ = nullptr;
    //@}

    //* list of refpoints
    RefPointWidget::List refPointWidgets_;

    //*@name transformation elements
    //@{

    //* origin
    QPointF origin_;

    //* matrix (a00, a01, a10, a11 )
    QVector<double> matrix_;

    //@}

};

#endif
