#ifndef UndoCommand_h
#define UndoCommand_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "RefPoint.h"
#include "MainWindow.h"
#include "WeakPointer.h"

#include <QTime>
#include <QApplication>
#include <QUndoCommand>

class UndoCommand: public QUndoCommand, private Base::Counter<UndoCommand>
{

    public:

    //* constructor
    explicit UndoCommand( MainWindow* parent ):
        Counter( "UndoCommand" ),
        parent_( parent ),
        time_( QTime::currentTime() )
    {}

    //*@name accessors
    //@{

    //* time
    const QTime& time() const
    { return time_; }

    //@}

    //* time
    void setTime( const QTime& time )
    { time_ = time; }

    protected:

    enum Type
    {
        MarkerAdded,
        MarkerRemoved,
        MarkerChanged,
        MarkersMovedUp,
        MarkersMovedDown,
        RefPointAdded,
        RefPointRemoved,
        RefPointChanged

    };

    //* parent
    Base::WeakPointer<MainWindow> _parent() const
    { return parent_; }

    //* true if both commands can be merged
    bool _canMerge( const UndoCommand* other ) const
    { return time().msecsTo( other->time() ) < qApp->doubleClickInterval(); }

    private:

    //* parent
    Base::WeakPointer<MainWindow> parent_;

    //* creation time
    QTime time_;

};

//* base class for marker commands
class MarkerCommand: public UndoCommand
{

    public:

    //* constructor
    explicit MarkerCommand( MainWindow* parent, int index ):
        UndoCommand( parent ),
        dataSetIndex_( index ),
        refPointIndex_( -1 )
    {}

    protected:

    //* data set
    int _dataSetIndex() const
    { return dataSetIndex_; }

    //* refpoint index
    void _setRefPointIndex( int value )
    { refPointIndex_ = value; }

    //* refpoint index
    int _refPointIndex() const
    { return refPointIndex_; }

    private:

    //* dataset
    int dataSetIndex_;

    //* ref point index
    int refPointIndex_;

};

//* undo command for adding a marker
class MarkerAddedCommand: public MarkerCommand
{

    public:

    //* constructor
    explicit MarkerAddedCommand( MainWindow* parent, int index, const RefPoint& point ):
        MarkerCommand( parent, index ),
        point_( point )
    { setText( QObject::tr( "Add Marker" ) ); }

    //* id
    int id() const override
    { return MarkerAdded; }

    //* undo
    void undo() override;

    //* redo
    void redo() override;

    private:

    //* source point
    RefPoint point_;

};

//* undo command for removing a marker
class MarkerRemovedCommand: public MarkerCommand
{

    public:

    //* constructor
    explicit MarkerRemovedCommand( MainWindow* parent, int dataSetIndex, const RefPoint& point ):
        MarkerCommand( parent, dataSetIndex ),
        point_( point )
    { setText( QObject::tr( "Remove Marker" ) ); }

    //* constructor
    explicit MarkerRemovedCommand( MainWindow* parent, int dataSetIndex, int refPointIndex, const RefPoint& point ):
        MarkerCommand( parent, dataSetIndex ),
        point_( point )
    {
        _setRefPointIndex( refPointIndex );
        setText( QObject::tr( "Remove Marker" ) );
    }

    //* id
    int id() const override
    { return MarkerRemoved; }

    //* undo
    void undo() override;

    //* redo
    void redo() override;

    private:

    //* source point
    RefPoint point_;

};

//* undo command for modifying a marker
class MarkerChangedCommand: public MarkerCommand
{

    public:

    //* constructor
    explicit MarkerChangedCommand( MainWindow* parent, int index, const RefPoint& source, const RefPoint& destination ):
        MarkerCommand( parent, index ),
        source_( source ),
        destination_( destination )
    { setText( QObject::tr( "Edit Marker" ) ); }

    //* id
    int id() const override
    { return MarkerChanged; }

    //* undo
    void undo() override;

    //* redo
    void redo() override;

    //* merge with
    bool mergeWith( const QUndoCommand* ) override;

    private:

    //* source point
    RefPoint source_;

    //* destination point
    RefPoint destination_;

};

//* base class for marker commands
class MarkersMovedCommand: public UndoCommand
{

    public:

    //* constructor
    explicit MarkersMovedCommand( MainWindow* parent, int index, const RefPoint::List& points ):
        UndoCommand( parent ),
        dataSetIndex_( index ),
        points_( points )
    {}

    protected:

    //* move up
    void _moveUp();

    //* move down
    void _moveDown();

    private:

    //* dataset
    int dataSetIndex_;

    //* ref point list
    RefPoint::List points_;

};

//* move markers up
class MarkersMovedUpCommand: public MarkersMovedCommand
{

    public:

    //* constructor
    explicit MarkersMovedUpCommand( MainWindow* parent, int index, const RefPoint::List& points ):
        MarkersMovedCommand( parent, index, points )
    { setText( QObject::tr( "Move Selection Up" ) ); }

    //* id
    int id() const override
    { return MarkersMovedUp; }

    //* undo
    void undo() override
    { _moveDown(); }

    //* undo
    void redo() override
    { _moveUp(); }

};

//* move markers down
class MarkersMovedDownCommand: public MarkersMovedCommand
{

    public:

    //* constructor
    explicit MarkersMovedDownCommand( MainWindow* parent, int index, const RefPoint::List& points ):
        MarkersMovedCommand( parent, index, points )
    { setText( QObject::tr( "Move Selection Down" ) ); }

    //* id
    int id() const override
    { return MarkersMovedDown; }

    //* undo
    void undo() override
    { _moveUp(); }

    //* undo
    void redo() override
    { _moveDown(); }

};

//* base class for reference points commands
class RefPointCommand: public UndoCommand
{

    public:

    //* constructor
    explicit RefPointCommand( MainWindow* parent, int index ):
        UndoCommand( parent ),
        index_( index )
    {}

    protected:

    //* reference point index
    int _index() const
    { return index_; }

    private:

    //* reference point index
    int index_;

};

//* undo command for adding a reference point
class RefPointAddedCommand: public RefPointCommand
{
    public:

    //* constructor
    explicit RefPointAddedCommand( MainWindow* parent, int index, const RefPoint& point ):
        RefPointCommand( parent, index ),
        point_( point )
    { setText( QObject::tr( "Add Reference Point" ) ); }

    //* id
    int id() const override
    { return RefPointAdded; }

    //* undo
    void undo() override;

    //* redo
    void redo() override;

    private:

    //* point
    RefPoint point_;

};

//* undo command for removing a reference point
class RefPointRemovedCommand: public RefPointCommand
{
    public:

    //* constructor
    explicit RefPointRemovedCommand( MainWindow* parent, int index, const RefPoint& point ):
        RefPointCommand( parent, index ),
        point_( point )
    { setText( QObject::tr( "Remove Reference Point" ) ); }

    //* id
    int id() const override
    { return RefPointRemoved; }

    //* undo
    void undo() override;

    //* redo
    void redo() override;

    private:

    //* point
    RefPoint point_;

};

//* undo command for adding a reference point
class RefPointChangedCommand: public RefPointCommand
{
    public:

    //* constructor
    explicit RefPointChangedCommand( MainWindow* parent, int index, const RefPoint& source, const RefPoint& destination ):
        RefPointCommand( parent, index ),
        source_( source ),
        destination_( destination )
    { setText( QObject::tr( "Edit Reference Point" ) ); }

    //* id
    int id() const override
    { return RefPointRemoved; }

    //* undo
    void undo() override;

    //* redo
    void redo() override;

    //* merge with
    bool mergeWith( const QUndoCommand* ) override;

    private:

    //* source
    RefPoint source_;

    //* source
    RefPoint destination_;

};

#endif
