/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ZoomWidget.h"

#include "CustomComboBox.h"
#include "CustomToolBar.h"
#include "IconEngine.h"
#include "IconNames.h"
#include "IconSize.h"
#include "Singleton.h"
#include "XmlOptions.h"

#include <QLayout>
#include <QLabel>
#include <QToolBar>

#include <cmath>

static const int nZoomFactors_ = 10;
static QStringList modes_;
static const double zoomFactors_[] = { 0.12, 0.25, 0.33, 0.50, 0.66, 0.75, 1.0, 1.25, 1.50, 2.0 };

//______________________________________________________________________________
ZoomWidget::ZoomWidget( QWidget* parent ):
    QWidget( parent ),
    Counter( "ZoomWidget" )
{

    // setup modes
    if( modes_.empty() )
    { modes_ << tr( "Fit Width" ) << tr( "Fit Height" ) << tr( "Fit Page" ); }

    Debug::Throw( "ZoomWidget::ZoomWidget" );
    QHBoxLayout* layout( new QHBoxLayout );
    layout->setMargin(0);
    setLayout( layout );

    QLabel* label;
    layout->addWidget( label = new QLabel( tr( "Zoom:" ), this ) );
    layout->addWidget( combobox_ = new CustomComboBox( this ) );
    label->setBuddy( combobox_ );

    combobox_->setEditable( true );
    combobox_->setDuplicatesEnabled( false );
    connect( combobox_->lineEdit(), SIGNAL(returnPressed()), SLOT(_setupItems()) );
    connect( combobox_, SIGNAL(currentIndexChanged(QString)), SLOT(_currentIndexChanged(QString)) );

    // setup items
    _setupItems( true );

    //! actions
    addAction( zoomOutAction_ = new QAction( IconEngine::get( IconNames::ZoomOut ), tr( "Zoom Out" ), this ) );
    zoomOutAction_->setShortcut( QKeySequence::ZoomOut );
    connect( zoomOutAction_, SIGNAL(triggered()), SLOT(_zoomOut()) );

    addAction( zoomInAction_ = new QAction( IconEngine::get( IconNames::ZoomIn ), tr( "Zoom In" ), this ) );
    zoomInAction_->setShortcut( QKeySequence::ZoomIn );
    connect( zoomInAction_, SIGNAL(triggered()), SLOT(_zoomIn()) );

    CustomToolBar* toolbar( new CustomToolBar( tr( "Zoom" ), this ) );
    toolbar->setTransparent( true );
    toolbar->setMovable( false );
    toolbar->setToolButtonStyle(Qt::ToolButtonIconOnly );
    toolbar->setIconSize( IconSize::get( IconSize::Small ) );
    toolbar->addAction( zoomOutAction_ );
    toolbar->addAction( zoomInAction_ );
    layout->addWidget( toolbar );
    layout->addStretch();
    connect( Base::Singleton::get().application(), SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    _updateConfiguration();

}

//______________________________________________________________________________
bool ZoomWidget::eventFilter( QObject*, QEvent* event )
{

    switch( event->type() )
    {

        case QEvent::Wheel:
        {
            QWheelEvent* wheelEvent( static_cast<QWheelEvent*>( event ) );
            if( wheelEvent->modifiers() != Qt::ShiftModifier ) break;

            const int numDegrees = wheelEvent->delta() / 8;
            const int numSteps = numDegrees / 15;
            if( numSteps == 0 ) break;
            else if( numSteps > 0 ) _zoomOut( numSteps );
            else _zoomIn( -numSteps );
            break;

        }

        default: break;

    }

    return false;

}

//______________________________________________________________________________
void ZoomWidget::realFactorChanged( double value )
{
    Debug::Throw() << "ZoomWidget::realFactorChanged - " << value << endl;
    realFactor_ = value;
    zoomOutAction_->setEnabled( realFactor_ > zoomFactors_[0] );
}

//______________________________________________________________________________
void ZoomWidget::_zoomIn( int steps )
{

    Debug::Throw() << "ZoomWidget::_ZoomIn -"
        << " factor: " << factor_
        << " realFactor: " << realFactor_
        << endl;

    const double factor( mode_ == Fixed ? factor_ : realFactor_ );
    int index( _findNearestFactor( factor ) );

    // decrement steps when exiting fixed mode
    if( mode_ != Fixed && index < nZoomFactors_ && index >= 0 && zoomFactors_[index] != realFactor_ )
    { steps--; }

    if( steps <= 0 )
    {
        _setFactor( zoomFactors_[index] );
        return;
    }

    // index in range
    for( int step = 0; step < steps; ++step )
    {

        if( index >= 0 && index < nZoomFactors_-1 )
        {

            _setFactor( zoomFactors_[index+1] );
            index++;

        } else {

            // calculate nearest higher zoom
            double newFactor = std::ceil(realFactor_*2)/2.0 + 0.5;
            QString buffer;
            QTextStream( &buffer ) << newFactor*100 << "%";
            combobox_->setEditText( buffer );
            _setupItems();

        }

    }

    return;
}

//______________________________________________________________________________
void ZoomWidget::_zoomOut( int steps )
{

    Debug::Throw() << "ZoomWidget::_ZoomOut -"
        << " factor: " << factor_
        << " realFactor: " << realFactor_
        << " steps: " << steps
        << endl;

    const double factor( mode_ == Fixed ? factor_ : realFactor_ );
    int index( _findNearestFactor( factor ) );

    for( int step = 0; step < steps; step++ )
    {

        if( index > 0 && index < nZoomFactors_ )
        {

            _setFactor( zoomFactors_[index-1] );
            index = qMax( index-1, 0 );

        } else if( index < 0 ) {

            double newFactor = std::ceil( realFactor_*2 )/2.0 - 0.5;
            QString buffer;
            QTextStream( &buffer ) << newFactor*100 << "%";
            combobox_->setEditText( buffer );
            _setupItems();
            _currentIndexChanged( buffer );

        }

    }

    return;
}

//______________________________________________________________________________
void ZoomWidget::_setupItems( bool forced )
{

    Debug::Throw( "ZoomWidget::_setupItems.\n" );

    // initialize static list
    static QStringList staticItems;
    if( staticItems.isEmpty() )
    {

        for( int index = 0; index < nZoomFactors_; ++index )
        {
            QString buffer;
            QTextStream( &buffer ) << 100*zoomFactors_[index] << "%";
            staticItems << buffer;
        }

    }

    // get current text
    bool currentIsNew( false );
    bool currentIsValid( false );
    double currentValue( 0 );
    QString currentBuffer( combobox_->currentText() );
    if( !staticItems.contains( currentBuffer ) )
    {

        currentIsNew = true;

        // try build item from current buffer
        currentBuffer.replace( '%', ' ' );
        currentValue = currentBuffer.toDouble();
        if( currentValue > 0 )
        {
            currentIsValid = true;
            currentBuffer.clear();
            QTextStream( &currentBuffer ) << currentValue << "%";
        }

    } else if( !forced ) return;

    QStringList items( staticItems );
    if( currentIsNew && currentIsValid )
    {
        bool inserted( false );
        for( auto&& iter = items.begin(); iter != items.end(); ++iter )
        {
            if( QString( *iter ).replace( '%', ' ' ).toDouble() > currentValue )
            {
                items.insert( iter, currentBuffer );
                inserted = true;
                break;
            }
        }

        if( !inserted ) items << currentBuffer;

    }

    // update combobox items
    combobox_->clear();
    combobox_->addItems( modes_ );
    combobox_->addItems( items );

    if( currentIsNew && currentIsValid ) combobox_->setCurrentIndex( combobox_->findText( currentBuffer ) );
    else _setFactor( factor_ );

}

//______________________________________________________________________________
void ZoomWidget::_currentIndexChanged( const QString& value )
{
    Debug::Throw() << "ZoomWidget::_currentIndexChanged - " << value << " configured: " << configured_ << endl;

    // try find mode
    const int index( modes_.indexOf( value ) );
    if( index >= 0 )
    {
        if( mode_ != (Mode) index )
        {
            mode_ = (Mode) index;
            emit modified( mode_, factor_ );
        }

    } else {

        // try define new scale factor
        const double factor( QString( value ).replace( '%', ' ' ).toDouble()/100 );
        if( factor > 0 && ( factor_ != factor || mode_ != Fixed ) )
        {
            mode_ = Fixed;
            factor_ = factor;
            realFactor_ = factor;
            emit modified(  mode_, factor_ );
        }

    }

    if( configured_ )
    {
        XmlOptions::get().set<int>( "ZOOM_MODE", mode_ );
        XmlOptions::get().set<double>( "ZOOM_FACTOR", factor_ );
        XmlOptions::write();
    }

    return;

}

//______________________________________________________________________________
void ZoomWidget::_setFactor( double value )
{

    // check value
    if( value <= 0 ) value = zoomFactors_[0];

    // construct string
    QString buffer;
    QTextStream( &buffer ) << 100*value << "%";
    combobox_->setCurrentIndex( combobox_->findText( buffer ) );

}

//______________________________________________________________________________
int ZoomWidget::_findNearestFactor( double value ) const
{

    for( int index = 0; index < nZoomFactors_; ++index )
    { if( zoomFactors_[index] >= value ) return index; }

    return -1;
}

//______________________________________________________________________________
void ZoomWidget::_updateConfiguration()
{
    Debug::Throw( "ZoomWidget::_updateConfiguration.\n" );
    Mode mode = (Mode) XmlOptions::get().get<int>( "ZOOM_MODE" );
    double factor = XmlOptions::get().get<double>( "ZOOM_FACTOR" );

    if( mode < Fixed )
    {

        combobox_->setCurrentIndex( mode );
        _currentIndexChanged( modes_[mode] );

    } else _setFactor( factor );

    configured_ = true;

}
